package com.sztzjy.forex.trading_trading.annotation.aspect;

import com.sztzjy.forex.trading_trading.annotation.Permission;
import com.sztzjy.forex.trading_trading.config.security.JwtUser;
import com.sztzjy.forex.trading_trading.config.security.TokenProvider;
import com.sztzjy.forex.trading_trading.service.RoleAuthorityService;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.*;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.core.Ordered;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Method;
import java.util.List;

/**
 * 系统权限拦截控制切面实现
 *
 * @author 陈沅
 */
@Aspect
@Component
public class PermissionAspect implements Ordered{

    @Resource
    private HttpServletRequest request;

    @Resource
    private RoleAuthorityService roleAuthorityService;

    @Pointcut("@annotation(com.sztzjy.forex.trading_trading.annotation.Permission)")
    public void authPointcut() {
    }


    @Around("authPointcut()")
    public Object around(ProceedingJoinPoint point) throws Throwable {
        Object result = point.proceed();
        return result;
    }


    @Before("authPointcut()")
    public void before(JoinPoint joinPoint) {
        JwtUser onlineUser = TokenProvider.getJWTUser(request);
        if (onlineUser == null) {
            throw new AccessDeniedException("当前用户未登录");
        }
        // 获取注解参数
        MethodSignature sign = (MethodSignature) joinPoint.getSignature();
        Method method = sign.getMethod();
        Permission permission = method.getAnnotation(Permission.class);
        //限制权限类型
        String[] codes = permission.codes();
        if(onlineUser.getRoleId()==0){
            throw new AccessDeniedException("权限不足");
        }
        //角色绑定的权限列表
        List<String> codeList = roleAuthorityService.getAuthorityByRoleId(onlineUser.getRoleId());
        boolean hasPermission = false;
        for (String code : codes){
            if(codeList.contains(code)){
                hasPermission=true;
                break;
            }
        }
        if(!hasPermission){
            throw new AccessDeniedException("权限不足");
        }
    }


    @After("authPointcut()")
    public void after() {
        //log.info("======>>> 后置通知：执行结束");
    }

    @Override
    public int getOrder() {
        return Ordered.HIGHEST_PRECEDENCE + 1;
    }
}
