package com.sztzjy.forex.trading_trading.controller;


import com.sztzjy.forex.trading_trading.annotation.Permission;
import com.sztzjy.forex.trading_trading.annotation.aspect.PermissionType;
import com.sztzjy.forex.trading_trading.config.security.JwtUser;
import com.sztzjy.forex.trading_trading.config.security.TokenProvider;
import com.sztzjy.forex.trading_trading.dto.GradeWeightBO;
import com.sztzjy.forex.trading_trading.entity.GradeWeight;
import com.sztzjy.forex.trading_trading.service.GradeWeightService;
import com.sztzjy.forex.trading_trading.util.ResultEntity;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.util.Assert;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

@Api(tags = "教师端：成绩权重管理")
@RestController
@RequestMapping("api/gradeWeight")
@RequiredArgsConstructor
public class GradeWeightController {

    private final GradeWeightService gradeWeightService;
    private final HttpServletRequest request;

    @Permission(codes = PermissionType.TRAINING_MANAGEMENT_ADD)
    @ApiOperation("教师端----新增一条成绩权重记录")
    @PostMapping("add")
    public ResultEntity add(@RequestBody GradeWeightBO bo) {
        Assert.notNull(bo, "参数错误");
        JwtUser user = TokenProvider.getJWTUser(request);
        gradeWeightService.insert(bo,user);
        return new ResultEntity(HttpStatus.OK);
    }


    @Permission(codes = PermissionType.TRAINING_MANAGEMENT_SEARCH)
    @ApiOperation("教师端----查询当前使用的成绩权重设置")
    @GetMapping("findCurrentWeight")
    public ResultEntity<List<GradeWeight>> findCurrentWeight() {
        JwtUser currentUser = TokenProvider.getJWTUser(request);
        List<GradeWeight> gradeWeights = gradeWeightService.findWeightBySchoolId(currentUser.getSchoolId());
        return new ResultEntity<List<GradeWeight>>(gradeWeights);
    }

}
