package com.sztzjy.forex.trading_trading.controller;

import com.github.pagehelper.PageInfo;
import com.sztzjy.forex.trading_trading.config.security.JwtUser;
import com.sztzjy.forex.trading_trading.config.security.TokenProvider;
import com.sztzjy.forex.trading_trading.entity.Member;
import com.sztzjy.forex.trading_trading.mappers.MemberMapper;
import com.sztzjy.forex.trading_trading.mappers.TrainingMapper;
import com.sztzjy.forex.trading_trading.service.IndexService;
import com.sztzjy.forex.trading_trading.service.TrainingService;
import com.sztzjy.forex.trading_trading.util.ResultEntity;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.List;

@Api(tags = "首页")
@RestController
@RequestMapping("api/index")
@RequiredArgsConstructor
public class IndexController {
    private final HttpServletRequest request;

    private final IndexService indexService;
    private final TrainingService trainingService;
    private final MemberMapper memberMapper;
    private final TrainingMapper trainingMapper;

    @GetMapping("/findAll")
    @ApiOperation("根据条件查询成员信息(分页)")
//    @AnonymousAccess
    public ResultEntity<PageInfo<Member>> findAll(@ApiParam("分页索引：{0}为第一页") Integer index,
                                                  @ApiParam("页量") @RequestParam Integer size,
                                                  @ApiParam("班级") @RequestParam(required = false) String classGrade,
                                                  @ApiParam("学号") @RequestParam(required = false) String studentNumber,
                                                  @ApiParam("姓名") @RequestParam(required = false) String name,
                                                  @ApiParam("日期") @RequestParam(required = false) @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss") Date startTime,
                                                  @ApiParam("实训任务名字") @RequestParam(required = false) String trainingName
    ) {
        JwtUser currentUser = TokenProvider.getJWTUser(request);
        int schoolId = currentUser.getSchoolId();
        PageInfo<Member> byConditions = indexService.findByConditions(index, size, classGrade, studentNumber, name, startTime, trainingName, schoolId);
        return new ResultEntity<>(byConditions);
    }

//    //首页下拉获取实训任务名称
//    @GetMapping("/getTrainingName")
//    @ApiOperation("首页下拉获取实训任务名称")
//    @AnonymousAccess
//    public ResultEntity<List<String>> getTrainingName() {
////        List<String> trainingName = trainingMapper.selectTrainingName();
////        return new ResultEntity<>(HttpStatus.OK, trainingName);
//        return new ResultEntity<List<String>>(HttpStatus.OK);
//    }

    //首页下拉获取班级
    @GetMapping("/getClassGrade")
    @ApiOperation("首页下拉获取班级")
//    @AnonymousAccess
    public ResultEntity<List<String>> getClassGrade() {
        //获取当前登录用户的信息。和@AnonymousAccess互斥
        JwtUser currentUser = TokenProvider.getJWTUser(request);
        int schoolId = currentUser.getSchoolId();
        List<String> classGrades = memberMapper.selectClassGrade(schoolId);
        return new ResultEntity<>(HttpStatus.OK, classGrades);
    }
}
