package com.sztzjy.forex.trading_trading.controller;

import com.sztzjy.forex.trading_trading.annotation.AnonymousAccess;
import com.sztzjy.forex.trading_trading.config.security.JwtUser;
import com.sztzjy.forex.trading_trading.dto.PageVO;
import com.sztzjy.forex.trading_trading.entity.Log;
import com.sztzjy.forex.trading_trading.service.LogService;
import com.sztzjy.forex.trading_trading.util.ResultEntity;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import org.springframework.data.domain.Page;
import org.springframework.http.HttpStatus;
import org.springframework.security.core.session.SessionInformation;
import org.springframework.security.core.session.SessionRegistry;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;

@Api(tags = "系统：操作日志")
@RestController
@RequestMapping("api/syslog")
@RequiredArgsConstructor
public class LogController {
    private final LogService logService;

    @ApiOperation("根据条件查询操作日志(分页)")
    @GetMapping("findAll")
    @AnonymousAccess
    public ResultEntity<PageVO<Log>> findAll(@ApiParam("ip地址") @RequestParam(required = false) String ipAddress,
                                             @ApiParam("开始时间") @RequestParam(required = false) Long startTime,
                                             @ApiParam("结束时间") @RequestParam(required = false) Long endTime,
                                             @ApiParam("分页索引：{0}为第一页") @RequestParam(required = false) Integer index,
                                             @ApiParam("页量") @RequestParam(required = false) Integer size,
                                             @ApiParam("操作人") @RequestParam(required = false) String operatorName
    ) {
        Page<Log> page = logService.findByConditions(index, size, ipAddress, startTime, endTime, operatorName);
        PageVO<Log> pageVO = new PageVO<>();
        pageVO.setPageInfo(page, page.getContent());
        return new ResultEntity<>(HttpStatus.OK, pageVO);
    }


}
