package com.sztzjy.forex.trading_trading.controller;

import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageInfo;
import com.sztzjy.forex.trading_trading.annotation.AnonymousAccess;
import com.sztzjy.forex.trading_trading.annotation.Permission;
import com.sztzjy.forex.trading_trading.annotation.aspect.PermissionType;
import com.sztzjy.forex.trading_trading.config.Constant;
import com.sztzjy.forex.trading_trading.config.security.JwtUser;
import com.sztzjy.forex.trading_trading.config.security.TokenProvider;
import com.sztzjy.forex.trading_trading.dto.MemberVO;
import com.sztzjy.forex.trading_trading.entity.ForexMarketData;
import com.sztzjy.forex.trading_trading.entity.Member;
import com.sztzjy.forex.trading_trading.entity.TakeStash;
import com.sztzjy.forex.trading_trading.entity.Training;
import com.sztzjy.forex.trading_trading.service.GradeWeightService;
import com.sztzjy.forex.trading_trading.service.MemberService;
import com.sztzjy.forex.trading_trading.service.TakeStashService;
import com.sztzjy.forex.trading_trading.service.TrainingService;
import com.sztzjy.forex.trading_trading.util.BigDecimalUtils;
import com.sztzjy.forex.trading_trading.util.ForexMarketDateUtil;
import com.sztzjy.forex.trading_trading.util.ResultEntity;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.FileNotFoundException;
import java.util.List;


@Api(tags = "实训任务用户管理")
@RestController
@RequestMapping("api/member")
public class MemberController {

    @Autowired
    MemberService memberService;
    @Autowired
    GradeWeightService gradeWeightService;
    @Autowired
    TrainingService trainingService;

    @Autowired
    TakeStashController takeStashController;

    @Autowired
    BigDecimalUtils bigDecimalUtils;

    @Autowired
    private HttpServletRequest request;

    @Autowired
    TakeStashService takeStashService;

    @Autowired
    ForexMarketDateUtil forexMarketDateUtil;

    @AnonymousAccess
    @PostMapping("getMemberById")
    public ResultEntity getMember(@RequestBody String memberId) {
        Member member = memberService.selectByPrimaryKey(memberId);
        Double availableFunds = getAvailableFunds(member);
        member.setAvailableFunds(availableFunds);
        return new ResultEntity(HttpStatus.OK, "根据成员ID获取成员对象", member);
    }

    @AnonymousAccess
    @PostMapping("getMember")
    public ResultEntity getMemberId(@RequestBody JSONObject jsonObject) {
        String name = jsonObject.getString("name");
        String trainingId = jsonObject.getString("trainingId");
        Integer schoolId =999999999;
        if(!"999999999".equals(trainingId)){
            schoolId = jsonObject.getInteger("schoolId");
        }
        Member member = memberService.selectByNameAndSchoolIdAndTrainingID(name, schoolId, trainingId);

        Double positionProfitLoss = takeStashController.flashTotalPositionProfitLoss(member.getMemberId());
        if (positionProfitLoss==null){
            positionProfitLoss=0.0;
        }
        member.setPositionProfitLoss(positionProfitLoss);  //设置持仓盈亏
        Double initialCapital = member.getInitialCapital();//初始资金
        Double cumulativeProfitLoss = member.getCumulativeProfitLoss(); //累计盈亏
        if(cumulativeProfitLoss==null){
            member.setCumulativeProfitLoss(0.0);
        }
        Double totalAssets = initialCapital+cumulativeProfitLoss;  //总资产=初始资金+累计盈亏
        Double netValue = bigDecimalUtils.add(totalAssets, positionProfitLoss);
        member.setTotalAssets(totalAssets);  //设置总资产
        member.setNetValue(netValue); //设置净值

        Double marginUsed = member.getMarginUsed(); //获取已用保证金
        if (marginUsed==null || marginUsed==0.0){
            marginUsed=0.0;
            member.setMarginLevel(null);
            member.setMarginUsed(0.0);
        }else {
            member.setMarginLevel(bigDecimalUtils.div(netValue, marginUsed, 2)); //设置保证金水平
        }
        member.setAvailableFunds(bigDecimalUtils.sub(netValue, marginUsed));  //设置可用资金

        return new ResultEntity(HttpStatus.OK, "获取成员ID", member);
    }

    @Permission(codes = PermissionType.TRAINING_MANAGEMENT_SEARCH)
    @ApiOperation("教师端----首页用户操作数据展示（分页）")
    @GetMapping("pagedListMembers")
    public ResultEntity<PageInfo<MemberVO>> pagedListMembers(@ApiParam("班级名称") String className,
                                                             @ApiParam("实训名称") String trainingName,
                                                             @ApiParam("评分状态") Integer markStatus,
                                                             @ApiParam("查询关键字：学号或学生名称") String keyword,
                                                             @ApiParam("分页索引：{0}为第一页") @RequestParam(required = false) Integer index,
                                                             @ApiParam("页量") @RequestParam(required = false) Integer size) {

        JwtUser jwtUser = TokenProvider.getJWTUser(request);
        PageInfo<MemberVO> pageInfo = memberService.pagedListMembers(className, trainingName, markStatus, keyword, jwtUser.getSchoolId(),index, size);
        if (!pageInfo.getList().isEmpty()) {
            for (MemberVO vo : pageInfo.getList()) {
                Training training = trainingService.findById(vo.getTrainingId());
                if (vo.getYield() == null || training == null) continue;
                vo.setTradeTrainingScore(gradeWeightService.findWeightByWeightIdAndYield(training.getWeightId(), vo.getYield()).getActualScore());
            }
        }
        return new ResultEntity<PageInfo<MemberVO>>(pageInfo);
    }

    @AnonymousAccess
    @ApiOperation("教师端首页排行榜数据导出")
    @GetMapping("exportMembers")
    public void exportMembers(@ApiParam("班级名称") String className,
                              @ApiParam("实训名称") String trainingName,
                              @ApiParam("评分状态") Integer markStatus,
                              @ApiParam("查询关键字：学号或学生名称") String keyword,
                              String TOKEN,
                              HttpServletResponse response) throws FileNotFoundException {
        TokenProvider.getJWTUser(TOKEN);
        memberService.exportMembers(className, trainingName, markStatus, keyword, response);
    }

    @Permission(codes = PermissionType.TRAINING_MANAGEMENT_SEARCH)
    @ApiOperation("根据实训任务id获取成员列表")
    @GetMapping("listMembersByTrainingId")
    public ResultEntity<PageInfo<Member>> listMembersByTrainingId(@RequestParam String trainingId,
                                                                  @ApiParam("班级id") Integer classId,
                                                                  @ApiParam("查询关键字：学号和姓名")String keyword,
                                                                  @RequestParam Integer index,
                                                                  @RequestParam Integer size) {
        PageInfo<Member> members = memberService.pageListMemberByTrainingId(trainingId, classId,keyword,index, size);
        return new ResultEntity<PageInfo<Member>>(members);
    }

    //获取可用资金
    public Double getAvailableFunds(Member member){
        Double positionProfitLoss = flashTotalPositionProfitLoss(member.getMemberId());
        if (positionProfitLoss==null){
            positionProfitLoss=0.0;
        }
        Double initialCapital = member.getInitialCapital();//初始资金
        Double totalAssets = initialCapital+positionProfitLoss;
        Double netValue = bigDecimalUtils.add(totalAssets, positionProfitLoss);
        Double cumulativeProfitLoss = member.getCumulativeProfitLoss(); //累计盈亏
        if(cumulativeProfitLoss==null){
            member.setCumulativeProfitLoss(0.0);
        }
        Double marginUsed = member.getMarginUsed(); //获取已用保证金
        if (marginUsed==null){
            marginUsed=0.0;
        }
        return bigDecimalUtils.sub(netValue, marginUsed);
    }

    public Double flashTotalPositionProfitLoss(String memberId) {
        List<TakeStash> takeStashList = takeStashService.selectAllByMemberIdAndStatus(memberId, 0);
        Double totalProfitAndLoss = 0.0;
        for (int i = 0; i < takeStashList.size(); i++) {
            TakeStash takeStash = takeStashList.get(i);
            String buySellType = takeStash.getBuySellType();  //买入或卖出
            String tradingCode = takeStash.getTradingCode();  //交易品种
            Double priceTransaction = takeStash.getPriceTransaction();  //交易价格
            Double volumeTransaction = takeStash.getVolumeTransaction(); //交易量
            ForexMarketData forexData = forexMarketDateUtil.getForexMarketDateByCode(tradingCode);
            Double nowBuyPic = forexData.getBuyPic();  //当前买价
            Double nowSellPic = Double.valueOf(forexData.getSellPic());  //当前卖价
            Double profitAndLoss;
            if (tradingCode.startsWith("USD")) { //美元在前
                if ("buy".equals(buySellType)) { //买
                    profitAndLoss = (nowSellPic - priceTransaction) * volumeTransaction * Constant.LEVERQUANTITY / nowSellPic;
                } else { //卖
                    profitAndLoss = (priceTransaction - nowBuyPic) * volumeTransaction * Constant.LEVERQUANTITY / nowBuyPic;
                }
            } else {  //美元在后
                if ("buy".equals(buySellType)) { //买
                    profitAndLoss = (nowSellPic - priceTransaction) * volumeTransaction * Constant.LEVERQUANTITY;
                } else { //卖
                    profitAndLoss = (nowBuyPic - priceTransaction) * volumeTransaction * Constant.LEVERQUANTITY;
                }
            }
            totalProfitAndLoss = totalProfitAndLoss + profitAndLoss;
            totalProfitAndLoss = bigDecimalUtils.mul(totalProfitAndLoss, 1, 2);
        }
        return totalProfitAndLoss;
    }

}
