package com.sztzjy.forex.trading_trading.controller;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageInfo;
import com.sztzjy.forex.trading_trading.annotation.AnonymousAccess;
import com.sztzjy.forex.trading_trading.dto.PengdingVo;
import com.sztzjy.forex.trading_trading.dto.TakeStashVO;
import com.sztzjy.forex.trading_trading.entity.ForexMarketData;
import com.sztzjy.forex.trading_trading.entity.PendingOrder;
import com.sztzjy.forex.trading_trading.service.PendingOrderService;
import com.sztzjy.forex.trading_trading.util.ForexMarketDateUtil;
import com.sztzjy.forex.trading_trading.util.RedisUtil;
import com.sztzjy.forex.trading_trading.util.ResultEntity;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.*;

@RestController
@RequestMapping("api/pendingOrder")
public class PendingOrderController {
    @Autowired
    PendingOrderService pendingOrderService;

    @Autowired
    ForexMarketDateUtil forexMarketDateController;

    @Autowired
    RedisUtil redisUtil;

    @Autowired
    TradingController tradingController;

    @Autowired
    ForexMarketDateUtil forexMarketDateUtil;

    //查询挂单数据
    @AnonymousAccess
    @PostMapping("getPendingOrder")
    public ResultEntity<List<PengdingVo>> getPendingOrder(@RequestBody JSONObject jsonObject) {
        String trainingId = String.valueOf(jsonObject.get("trainingId"));
        String memberId = String.valueOf(jsonObject.get("memberId"));
        Integer pageIndex = jsonObject.getInteger("index");
        Integer pageSize = jsonObject.getInteger("size");
        PendingOrder pendingOrder = new PendingOrder();
        pendingOrder.setStatus(0);
        pendingOrder.setTrainingId(trainingId);
        pendingOrder.setMemberId(memberId);
        PageInfo<PendingOrder> pendingOrderPageInfo = pendingOrderService.selectByTrainingIdAndStatusAndMemberId(pendingOrder, pageIndex, pageSize);
        List<PendingOrder> pendingOrders = pendingOrderPageInfo.getList();
        List<PengdingVo> pengdingVoList = new ArrayList<>();
        for (int i = 0; i < pendingOrders.size(); i++) {
            PengdingVo pengdingVo = new PengdingVo(pendingOrders.get(i));
            pengdingVoList.add(pengdingVo);
        }
        List<TakeStashVO> data = flashPendingCurrentPrice(pengdingVoList).getBody().getData();
        PageInfo<TakeStashVO> pageInfo=new PageInfo<>();
        pageInfo.setTotal(pendingOrderPageInfo.getTotal());
        pageInfo.setList(data);
        return new ResultEntity(HttpStatus.OK, "获取挂单数据成功", pageInfo);
    }

    //撤单 撤单成功后 根据pendingOrderId删除redis键
    @AnonymousAccess
    @PostMapping("cancelOrder")
    public ResultEntity cancelOrder(@RequestBody JSONObject jsonObject) {
        String pendingOrderId = String.valueOf(jsonObject.get("pendingOrderId"));
        pendingOrderService.cancelOrder(pendingOrderId);
        redisUtil.del("pengingOrder_" + pendingOrderId);
        return new ResultEntity(HttpStatus.OK, "撤单成功");
    }



    //判断是否开仓
    public Boolean isOpenTakeStash(Map<Object, Object> pengingOrderMap,Map<String, ForexMarketData> forexMarketDataMap) {
        String buySellType = (String) pengingOrderMap.get("buySellType");
        String tradingCode = (String) pengingOrderMap.get("tradingCode");
        Double priceCommission = (Double) pengingOrderMap.get("priceCommission");
        ForexMarketData forexDate = forexMarketDataMap.get(tradingCode);
        Double buyPic = forexDate.getBuyPic();
        Double sellPic = Double.valueOf(forexDate.getSellPic());
        if ("buyLimit".equals(buySellType)) { //限价买进
            if (buyPic < priceCommission) {
                return true;
            }
        }
        if ("sellLimit".equals(buySellType)) { //限价卖出
            if (sellPic > priceCommission) {
                return true;
            }
        }
        if ("buyStop".equals(buySellType)) {  //止损买入
            if (buyPic > priceCommission) {
                return true;
            }
        }
        if ("sellStop".equals(buySellType)) {
            if (sellPic < priceCommission) {
                return true;
            }
        }
        return false;

    }

    //判断是否到期 到期则删除挂单数据和redis中的键
    public Boolean isExpire(Map<Object, Object> pengingOrderMap, String key) {
        Date validityTime = (Date) pengingOrderMap.get("validityTime"); //获取有效期
        Date currentDate = new Date();
        if (validityTime.after(currentDate)) {
            pendingOrderService.cancelOrder((String) pengingOrderMap.get("pendingOrderId"));
            redisUtil.del(key);
            return true;
        }
        return false;
    }

    public JSONObject pengingKeyReturnJSONObject(Map<Object, Object> pengingOrderMap) {
        Object tradingCode = pengingOrderMap.get("tradingCode");
        Object currencyName = pengingOrderMap.get("currencyName");
        Object transactionType = pengingOrderMap.get("transactionType");
        Object transactionVolume = pengingOrderMap.get("transactionVolume");
        Object buySellTypeObj = pengingOrderMap.get("buySellType");
        String buySellType= String.valueOf(buySellTypeObj);
        if(buySellType.contains("buy")){
            buySellType="buy";
        }else {
            buySellType="sell";
        }
        Object stopLoss = pengingOrderMap.get("stopLoss");
        Object stopWin = pengingOrderMap.get("stopWin");
        Object memberId = pengingOrderMap.get("memberId");
        Object trainingId = pengingOrderMap.get("trainingId");
        Object priceCommission = pengingOrderMap.get("priceCommission");
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("tradingCode", tradingCode);
        jsonObject.put("currencyName", currencyName);
        jsonObject.put("transactionType", transactionType);
        jsonObject.put("transactionVolume", transactionVolume);
        jsonObject.put("buySellType", buySellType);
        jsonObject.put("stopLoss", stopLoss);
        jsonObject.put("stopWin", stopWin);
        jsonObject.put("memberId", memberId);
        jsonObject.put("trainingId", trainingId);
        jsonObject.put("priceCommission", priceCommission);
        return jsonObject;
    }


    //挂单刷新当前价位
    @AnonymousAccess
    @PostMapping("flashPendingCurrentPrice")
    public ResultEntity<List<TakeStashVO>> flashPendingCurrentPrice(@RequestBody List<PengdingVo> pengdingVoList) {
        List<PengdingVo> pengdingVos = new ArrayList<>();
        for (int i = 0; i < pengdingVoList.size(); i++) {
            PengdingVo pengdingVo = pengdingVoList.get(i);
            if (0 != pengdingVo.getStatus()) {
                continue;
            }
            String buySellType = pengdingVo.getBuySellType();  //买入或卖出
            String tradingCode = pengdingVo.getTradingCode();  //交易品种
            ForexMarketData forexData = forexMarketDateController.getForexMarketDateByCode(tradingCode);
            Double nowBuyPic = forexData.getBuyPic();  //当前买价
            Double nowSellPic = Double.valueOf(forexData.getSellPic());  //当前卖价
            if (tradingCode.startsWith("USD")) { //美元在前
                if ("buy".equals(buySellType)) { //买
                    pengdingVo.setCurrentPrice(nowBuyPic);
                } else { //卖
                    pengdingVo.setCurrentPrice(nowSellPic);
                }
            } else {  //美元在后
                if ("buy".equals(buySellType)) { //买
                    pengdingVo.setCurrentPrice(nowBuyPic);
                } else { //卖
                    pengdingVo.setCurrentPrice(nowSellPic);
                }
            }
            pengdingVos.add(pengdingVo);
        }
        if (0 == pengdingVos.size()) {
            return new ResultEntity(HttpStatus.OK, "挂单数据为空", null);
        }
        return new ResultEntity(HttpStatus.OK, "挂单当前价格刷新", pengdingVos);
    }
}
