package com.sztzjy.forex.trading_trading.controller;

import com.sztzjy.forex.trading_trading.annotation.AnonymousAccess;
import com.sztzjy.forex.trading_trading.annotation.Permission;
import com.sztzjy.forex.trading_trading.annotation.aspect.PermissionType;
import com.sztzjy.forex.trading_trading.config.security.JwtUser;
import com.sztzjy.forex.trading_trading.config.security.TokenProvider;
import com.sztzjy.forex.trading_trading.entity.Member;
import com.sztzjy.forex.trading_trading.entity.Report;
import com.sztzjy.forex.trading_trading.entity.ReportWithBLOBs;
import com.sztzjy.forex.trading_trading.service.MemberService;
import com.sztzjy.forex.trading_trading.service.ReportService;
import com.sztzjy.forex.trading_trading.util.ResultEntity;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

@Api(tags = "实验报告")
@RestController
@RequestMapping("api/report")
public class ReportController {
    @Autowired
    private ReportService reportService;
    @Autowired
    private MemberService memberService;

    @ApiOperation("学生端--上传实训报告")
    @PostMapping("/uploadReport")
    public ResultEntity uploadReport(@RequestParam("file")@RequestPart(name = "file") MultipartFile file,
                                             @RequestParam String memberId,
                                             @RequestParam String fileName) {
        Member member = memberService.selectByPrimaryKey(memberId);
        reportService.uploadReport(file, fileName, member);
        return new ResultEntity(HttpStatus.OK);
    }
    @ApiOperation("学生端--提交实训报告")
    @PostMapping("/submitReport")
    public ResultEntity<String> submitReport(@RequestBody ReportWithBLOBs report) {
        return new ResultEntity<>(reportService.submitReport(report));
    }


    @ApiOperation("教师端--对实训报告评分")
    @PostMapping("review")
    @Permission(codes = PermissionType.TRAINING_PLAN_MANAGEMENT_EDIT)
    public ResultEntity review(@RequestParam String id,
                               @RequestParam Double score) {
        reportService.review(id, score);
        return new ResultEntity(HttpStatus.OK);
    }

    @AnonymousAccess
    @ApiOperation("教师端--实训报告下载")
    @GetMapping("download")
    public void download(@RequestParam String id, String TOKEN,HttpServletResponse response) {
        TokenProvider.getJWTUser(TOKEN);
        reportService.download(id, response);
    }


    @ApiOperation("教师端--获取学生上传的报告")
    @GetMapping("getReportByMemberId")
    public ResultEntity<ReportWithBLOBs> getReportByMemberId(@RequestParam String memberId
    ) {
        return new ResultEntity<>(reportService.findByMemberId(memberId));
    }
}
