package com.sztzjy.forex.trading_trading.controller;


import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageInfo;
import com.sztzjy.forex.trading_trading.annotation.AnonymousAccess;
import com.sztzjy.forex.trading_trading.config.Constant;
import com.sztzjy.forex.trading_trading.dto.TakeStashVO;
import com.sztzjy.forex.trading_trading.entity.ForexMarketData;
import com.sztzjy.forex.trading_trading.entity.Member;
import com.sztzjy.forex.trading_trading.entity.TakeStash;
import com.sztzjy.forex.trading_trading.service.MemberService;
import com.sztzjy.forex.trading_trading.service.TakeStashService;
import com.sztzjy.forex.trading_trading.service.TrainingService;
import com.sztzjy.forex.trading_trading.service.WainingService;
import com.sztzjy.forex.trading_trading.util.BigDecimalUtils;
import com.sztzjy.forex.trading_trading.util.ForexMarketDateUtil;
import com.sztzjy.forex.trading_trading.util.RedisUtil;
import com.sztzjy.forex.trading_trading.util.ResultEntity;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.math.BigDecimal;
import java.util.*;

@RestController
@RequestMapping("api/takeStash")
public class TakeStashController {

    @Autowired
    TakeStashService takeStashService;

    @Autowired
    RedisUtil redisUtil;

    @Autowired
    TradingController tradingController;

    @Autowired
    MemberService memberService;

    @Autowired
    ForexMarketDateUtil forexMarketDateUtil;

    @Autowired
    TrainingService trainingService;

    @Autowired
    BigDecimalUtils bigDecimalUtils;

    @Autowired
    WainingService wainingService;

    //获取当前持仓表 status 0为获取当前持仓/1为挂单持仓/2为历史持仓
    @AnonymousAccess
    @PostMapping("getTakeStashList")
    public ResultEntity getTakeStashList(@RequestBody JSONObject jsonObject) {
        Integer pageIndex = jsonObject.getInteger("index");
        Integer pageSize = jsonObject.getInteger("size");
        TakeStash takeStash = jsonObject.getObject("takeStash", TakeStash.class);
        PageInfo<TakeStash> pageInfo = takeStashService.findTakeStashByTrainingIdAndMemberIdAndStatus(takeStash.getTrainingId(), takeStash.getMemberId(), takeStash.getStatus(), pageIndex, pageSize);
        List<TakeStash> takeStashList = pageInfo.getList();
        List<TakeStashVO> takeStashVOList = new ArrayList<>();
        for (int i = 0; i < takeStashList.size(); i++) {
            TakeStashVO takeStashVO = new TakeStashVO(takeStashList.get(i));
            takeStashVOList.add(takeStashVO);
        }
        if (takeStash.getStatus() == 0) {
            takeStashVOList = flashProfitAndLoss(takeStashVOList).getBody().getData();
        }
        PageInfo<TakeStashVO> voPageInfo = new PageInfo<>();
        voPageInfo.setTotal(pageInfo.getTotal());
        voPageInfo.setList(takeStashVOList);
        return new ResultEntity(HttpStatus.OK, "返回持仓数据", voPageInfo);
    }

    //根据持仓ID获取单个持仓数据
    @AnonymousAccess
    @PostMapping("getTakeStash")
    public TakeStash getTakeStash(@RequestBody JSONObject jsonObject) {
        String trainingId = String.valueOf(jsonObject.get("trainingId"));
        TakeStash takeStash = takeStashService.selectByPrimaryKey(trainingId);
        return takeStash;
    }

    //持仓盈亏和当前价格刷新
    @AnonymousAccess
    @PostMapping("flashProfitAndLoss")
    public ResultEntity<List<TakeStashVO>> flashProfitAndLoss(@RequestBody List<TakeStashVO> takeStashVOList) {
        List<TakeStashVO> takeStashVOS = new ArrayList<>();
        for (int i = 0; i < takeStashVOList.size(); i++) {
            TakeStashVO takeStashVO = takeStashVOList.get(i);
            if (0 != takeStashVO.getStatus()) {
                continue;
            }
            String buySellType = takeStashVO.getBuySellType();  //买入或卖出
            String tradingCode = takeStashVO.getTradingCode();  //交易品种
            Double priceTransaction = takeStashVO.getPriceTransaction();  //交易价格
            Double volumeTransaction = takeStashVO.getVolumeTransaction(); //交易量
            ForexMarketData forexData = forexMarketDateUtil.getForexMarketDateByCode(tradingCode);
            Double nowBuyPic = forexData.getBuyPic();  //当前买价
            Double nowSellPic = Double.valueOf(forexData.getSellPic());  //当前卖价
            Double profitAndLoss;
            if (tradingCode.startsWith("USD")) { //美元在前
                if ("buy".equals(buySellType)) { //买
                    profitAndLoss = (nowSellPic - priceTransaction) * volumeTransaction * Constant.LEVERQUANTITY / nowSellPic;
                    takeStashVO.setCurrentPrice(nowSellPic);
                } else { //卖
                    profitAndLoss = (priceTransaction - nowBuyPic) * volumeTransaction * Constant.LEVERQUANTITY / nowBuyPic;
                    takeStashVO.setCurrentPrice(nowBuyPic);
                }
            } else {  //美元在后
                if ("buy".equals(buySellType)) { //买
                    profitAndLoss = (nowSellPic - priceTransaction) * volumeTransaction * Constant.LEVERQUANTITY;
                    takeStashVO.setCurrentPrice(nowSellPic);
                } else { //卖
                    profitAndLoss = (nowBuyPic - priceTransaction) * volumeTransaction * Constant.LEVERQUANTITY;
                    takeStashVO.setCurrentPrice(nowBuyPic);
                }
            }
            takeStashVO.setProfitAndLoss(bigDecimalUtils.mul(profitAndLoss, 1, 2));
            takeStashVOS.add(takeStashVO);
        }
        if (0 == takeStashVOS.size()) {
            return new ResultEntity(HttpStatus.OK, "当前持仓数据为空", null);
        }
        return new ResultEntity(HttpStatus.OK, "持仓盈亏和当前价格刷新", takeStashVOS);
    }

    //总持仓盈亏刷新  获取member下所有持仓数据
    public Double flashTotalPositionProfitLoss(String memberId) {
        List<TakeStash> takeStashList = takeStashService.selectAllByMemberIdAndStatus(memberId, 0);
        Double totalProfitAndLoss = 0.0;
        for (int i = 0; i < takeStashList.size(); i++) {
            TakeStash takeStash = takeStashList.get(i);
            String buySellType = takeStash.getBuySellType();  //买入或卖出
            String tradingCode = takeStash.getTradingCode();  //交易品种
            Double priceTransaction = takeStash.getPriceTransaction();  //交易价格
            Double volumeTransaction = takeStash.getVolumeTransaction(); //交易量
            ForexMarketData forexData = forexMarketDateUtil.getForexMarketDateByCode(tradingCode);
            Double nowBuyPic = forexData.getBuyPic();  //当前买价
            Double nowSellPic = Double.valueOf(forexData.getSellPic());  //当前卖价
            Double profitAndLoss;
            if (tradingCode.startsWith("USD")) { //美元在前
                if ("buy".equals(buySellType)) { //买
                    profitAndLoss = (nowSellPic - priceTransaction) * volumeTransaction * Constant.LEVERQUANTITY / nowSellPic;
                } else { //卖
                    profitAndLoss = (priceTransaction - nowBuyPic) * volumeTransaction * Constant.LEVERQUANTITY / nowBuyPic;
                }
            } else {  //美元在后
                if ("buy".equals(buySellType)) { //买
                    profitAndLoss = (nowSellPic - priceTransaction) * volumeTransaction * Constant.LEVERQUANTITY;
                } else { //卖
                    profitAndLoss = (nowBuyPic - priceTransaction) * volumeTransaction * Constant.LEVERQUANTITY;
                }
            }
            totalProfitAndLoss = totalProfitAndLoss + profitAndLoss;
            totalProfitAndLoss = bigDecimalUtils.mul(totalProfitAndLoss, 1, 2);
        }
        return totalProfitAndLoss;
    }

    //修改当前持仓
    @AnonymousAccess
    @PostMapping("updateTakeStash")
    public ResultEntity updateTakeStash(@RequestBody JSONObject jsonObject) {
        String stashId = jsonObject.getString("stashId");
        Double stopLoss = jsonObject.getDouble("stopLoss");
        Double stopWin = jsonObject.getDouble("stopWin");
        String tradingCode = jsonObject.getString("tradingCode");
        String buySellType = jsonObject.getString("buySellType");
        ForexMarketData forexData = tradingController.getMarketQuotationByCode(tradingCode).getBody().getData();
        TakeStash takeStash = new TakeStash();
        takeStash.setStashId(stashId);
        if (null != stopLoss || null != stopWin) {   //判断止损止赢是否合理 如果stopLoss stopWin都为null则跳过
            boolean winOrLossStopBoolean = tradingController.getWinOrLossStop(stopLoss, stopWin, buySellType, forexData);
            if (winOrLossStopBoolean == false) {
                return new ResultEntity(HttpStatus.BAD_REQUEST, "止损或获利输入错误");
            }
            if (null != stopLoss) {
                takeStash.setStopLoss(stopLoss);
            }
            if (null != stopWin) {
                takeStash.setStopWin(stopWin);
            }
        }
        takeStashService.updateWinLossByPrimaryKey(takeStash);
        return new ResultEntity(HttpStatus.OK, "修改持仓成功");
    }


    //当前持仓平仓  status设置为2
    @AnonymousAccess
    @PostMapping("closeTakeStash")
    public ResultEntity closeTakeStash(@RequestBody JSONObject jsonObject) {
        String stashId = jsonObject.getString("stashId");
        String memberId = jsonObject.getString("memberId");
        TakeStash takeStashNew = takeStashService.selectByPrimaryKey(stashId);
        Member member = memberService.selectByPrimaryKey(memberId);
        String buySellType = takeStashNew.getBuySellType();
        String tradingCode = takeStashNew.getTradingCode();
        Double volumeTransaction = takeStashNew.getVolumeTransaction();
        Double priceTransaction = takeStashNew.getPriceTransaction();  //获取进仓时交易价格
        String trainingId = takeStashNew.getTrainingId();
        Double priceTransactionCloser = 0.0;  //记录出仓时交易价格
        ForexMarketData forexData = forexMarketDateUtil.getForexMarketDateByCode(tradingCode);
        Double profitAndLoss = 0.0; //盈亏
        Double nowSellPic = Double.valueOf(forexData.getSellPic());//当前卖价
        Double nowBuyPic = forexData.getBuyPic();  //当前买价
        Double margin=0.0;  //记录保证金变化量
        if (tradingCode.startsWith("USD")) {  //美元在前  返还资金表达式为
            //可用保证金计算
            margin = startUSDMarginNeed(trainingId,memberId,0,tradingCode,buySellType,volumeTransaction);
            if ("buy".equals(buySellType)) {  //如果持仓方向为买  则平仓方向为卖
                priceTransactionCloser = nowSellPic;
                profitAndLoss = (nowSellPic - priceTransaction) * volumeTransaction * Constant.LEVERQUANTITY / nowSellPic;
            } else if ("sell".equals(buySellType)) { //如果持仓方向为卖  则平仓方向为买 可用资金=可用资金+当前买价*1000*买卖手
                priceTransactionCloser = nowBuyPic;
                profitAndLoss = (priceTransaction - nowBuyPic) * volumeTransaction * Constant.LEVERQUANTITY / nowBuyPic;
            }
        } else { //美元在后
            margin =endUSDMarginNeed(trainingId,memberId,0,tradingCode,buySellType,takeStashNew.getVolumeTransaction(),priceTransaction);
            if ("buy".equals(buySellType)) {  //如果持仓方向为买  则平仓方向为卖 可用资金=可用资金+1000*买卖手
                priceTransactionCloser = nowSellPic;
                profitAndLoss = (nowSellPic - priceTransaction) * volumeTransaction * Constant.LEVERQUANTITY;
            } else if ("sell".equals(buySellType)) { //如果持仓方向为卖  则平仓方向为买 可用资金=可用资金+当前买价*1000*买卖手
                priceTransactionCloser = nowBuyPic;
                profitAndLoss = (nowBuyPic - priceTransaction) * volumeTransaction * Constant.LEVERQUANTITY;
            }
        }

        Double marginUsed = member.getMarginUsed();//未修改前member表中成员保证金
        marginUsed = bigDecimalUtils.mul(marginUsed-margin,1,2); //修改成员保证金
        member.setMarginUsed(marginUsed);
        Integer closingTrades = member.getClosingTrades();
        if(closingTrades==null){
            closingTrades=1;
        }
        member.setClosingTrades(closingTrades++);
        Double cumulativeProfitLoss = member.getCumulativeProfitLoss(); //为修改前member表中的累积盈亏
        member.setCumulativeProfitLoss(bigDecimalUtils.add(cumulativeProfitLoss, profitAndLoss));  //修改累计盈亏
        member.setTotalAssets(bigDecimalUtils.add(bigDecimalUtils.add(member.getInitialCapital(), cumulativeProfitLoss), profitAndLoss));  //设置总资产
        memberService.updateByPrimaryKeySelective(member);
        flashTotalPositionProfitLoss(member.getMemberId());


        TakeStash takeStash = new TakeStash();
        takeStash.setStashId(stashId);
        takeStash.setStatus(2);
        takeStash.setPriceTransactionClose(priceTransactionCloser);
        takeStash.setProfitAndLossByClose(bigDecimalUtils.mul(profitAndLoss,1,2));
        takeStash.setTimeTransactionClose(new Date());
        takeStashService.updateByPrimaryKeySelective(takeStash);

        wainingService.compareMarginLevels(member.getMemberId(), member.getTrainingId());//更改可用保证金后 调用预警

        return new ResultEntity(HttpStatus.OK, "平仓成功");
    }

    //美元在后  返回保证金变化量  公式（（sell交易量*交易价格+sell交易量*价格+...）/交易总量+(buy交易量*交易价格+buy交易量*价格+...)/交易总量）*合约量/杠杆倍数*sellbuy最高交易量
    public Double endUSDMarginNeed(String trainingId,String memberId,int status,String tradingCode,String buySellType,Double transactionVolume,Double priceTransaction){
        Double margin=0.0;
        //获取持仓表中tradingCode所有buysell情况  根据trainingId和memberId查询交易品种=tradingCode的所有数据
        List<TakeStash> takeStashList=takeStashService.findTakeStashByTrainingIdAndMemberIdAndStatusAndTradingCode(trainingId,memberId,status,tradingCode);
        Double buyTotalMargin=0.0;  //类型为买的 交易量*交易价格之和
        Double sellTotalMargin=0.0; //类型为卖的 交易量*交易价格之和
        Double totalVolumeTransaction=0.0; //交易总量
        Double buyTotalVolumeTransaction=0.0;  //buy类型交易总量
        Double sellTotalVolumeTransaction=0.0; //sell类型交易总量
        Double highVolumeTransaction=0.0; //记录buy/sell类型交易总量中较高的一个
        if(takeStashList.size()==1){
            if("buy".equals(buySellType)){
                margin=transactionVolume*priceTransaction*Constant.LEVERQUANTITY;
            }else {
                margin=transactionVolume*priceTransaction*Constant.LEVERQUANTITY;
            }
            return margin;
        }else {
            for (int i = 0; i < takeStashList.size(); i++) {
                TakeStash takeStash = takeStashList.get(i);
                String buySellTypeTakeStash = takeStash.getBuySellType();
                Double volumeTransactionTakeStash = takeStash.getVolumeTransaction(); //持仓交易量
                Double priceTransactionTakeStash = takeStash.getPriceTransaction(); //持仓交易价格
                if("buy".equals(buySellTypeTakeStash)){
                    buyTotalMargin=buyTotalMargin+volumeTransactionTakeStash*priceTransactionTakeStash;
                    buyTotalVolumeTransaction=buyTotalVolumeTransaction+volumeTransactionTakeStash;
                }else {
                    sellTotalMargin=sellTotalMargin+volumeTransactionTakeStash*priceTransactionTakeStash;
                    sellTotalVolumeTransaction=sellTotalVolumeTransaction+volumeTransactionTakeStash;
                }
                totalVolumeTransaction=totalVolumeTransaction+volumeTransactionTakeStash;
            }
        }

        if (buyTotalVolumeTransaction>=sellTotalVolumeTransaction){  //设置平仓前 sell\buy的最高交易交易总量
            highVolumeTransaction=buyTotalVolumeTransaction;
        }else {
            highVolumeTransaction=sellTotalVolumeTransaction;
        }
        //获取相同tradingCode持仓表总使用保证金
        Double totalMargin=(buyTotalMargin+sellTotalMargin)/totalVolumeTransaction*Constant.LEVERQUANTITY*highVolumeTransaction;
        //获取去除平仓数据后的相同tradingCode持仓表总使用保证金
        if("buy".equals(buySellType)){
            if (buyTotalVolumeTransaction-transactionVolume>=sellTotalVolumeTransaction){  //设置平仓后 sell\buy的最高交易交易总量
                highVolumeTransaction=buyTotalVolumeTransaction;
            }else {
                highVolumeTransaction=sellTotalVolumeTransaction;
            }
        }else {
            if ( sellTotalVolumeTransaction-transactionVolume>=buyTotalVolumeTransaction){  //设置平仓后 sell\buy的最高交易交易总量
                highVolumeTransaction=sellTotalVolumeTransaction;
            }else {
                highVolumeTransaction=buyTotalVolumeTransaction;
            }
        }
        Double totalMarginRemoveCloseTakeStash=0.0;
        totalMarginRemoveCloseTakeStash=(buyTotalMargin+sellTotalMargin-transactionVolume*priceTransaction)/(totalVolumeTransaction-transactionVolume)*Constant.LEVERQUANTITY*highVolumeTransaction;
        margin=totalMargin-totalMarginRemoveCloseTakeStash;
        return margin;
    }


    //美元在前  返回保证金变化量
    public Double startUSDMarginNeed(String trainingId,String memberId,int status,String tradingCode,String buySellType,Double transactionVolume){
        Double margin=0.0;
        //获取持仓表中tradingCode所有buysell情况  根据trainingId和memberId查询交易品种=tradingCode的所有数据
        List<TakeStash> takeStashList=takeStashService.findTakeStashByTrainingIdAndMemberIdAndStatusAndTradingCode(trainingId,memberId,status,tradingCode);
        Double buyTotalVolumeTransaction=0.0;
        Double sellTotalVolumeTransaction=0.0;
        if(takeStashList.size()==1){
            margin=transactionVolume*Constant.LEVERQUANTITY;
            return margin;
        }else {
            for (int i = 0; i < takeStashList.size(); i++) {
                TakeStash takeStash = takeStashList.get(i);
                String buySellTypeTakeStash = takeStash.getBuySellType();
                if("buy".equals(buySellTypeTakeStash)){
                    buyTotalVolumeTransaction=buyTotalVolumeTransaction+takeStash.getVolumeTransaction();
                }else {
                    sellTotalVolumeTransaction=sellTotalVolumeTransaction+takeStash.getVolumeTransaction();
                }
            }
        }
        if("buy".equals(buySellType)){
            if(buyTotalVolumeTransaction>=sellTotalVolumeTransaction){
                if(buyTotalVolumeTransaction-transactionVolume>=sellTotalVolumeTransaction){
                    margin=transactionVolume*Constant.LEVERQUANTITY;
                }else {
                    margin=(buyTotalVolumeTransaction-sellTotalVolumeTransaction)*Constant.LEVERQUANTITY;
                }
            }else {
                margin=0.0;
            }
        }else {
            if(sellTotalVolumeTransaction>=buyTotalVolumeTransaction){
                if(sellTotalVolumeTransaction-transactionVolume>=buyTotalVolumeTransaction){
                    margin=transactionVolume*Constant.LEVERQUANTITY;
                }else {
                    margin=(sellTotalVolumeTransaction-buyTotalVolumeTransaction)*Constant.LEVERQUANTITY;
                }
            }else {
                margin=0.0;
            }
        }
        return margin;
    }


    public void updateMemberAndTakeStash(String memberId, String stashId , Double CumulativeProfitLoss,Double margin) {  //margin平仓后保证金变化量
        Member member = memberService.selectByPrimaryKey(memberId);
        Double marginUsed = member.getMarginUsed();//未修改前member表中成员保证金

        marginUsed = bigDecimalUtils.sub(marginUsed, margin); //修改成员保证金
        member.setMarginUsed(bigDecimalUtils.mul(marginUsed,1,2));
        Integer closingTrades = member.getClosingTrades();
        member.setClosingTrades(closingTrades++);
        Double cumulativeProfitLoss = member.getCumulativeProfitLoss(); //为修改前member表中的累积盈亏
        member.setCumulativeProfitLoss(bigDecimalUtils.mul(bigDecimalUtils.add(cumulativeProfitLoss, CumulativeProfitLoss),1,2));  //修改累计盈亏
        memberService.updateByPrimaryKeySelective(member);

        TakeStash takeStash = new TakeStash();
        takeStash.setStashId(stashId);
        takeStash.setStatus(2);
        takeStash.setProfitAndLossByClose(bigDecimalUtils.mul(CumulativeProfitLoss,1,2));   //盈亏=返还资金-进仓时使用的保证金
        takeStashService.updateByPrimaryKeySelective(takeStash);

        wainingService.compareMarginLevels(member.getMemberId(), member.getTrainingId());//更改可用保证金后 调用预警
    }
}
