package com.sztzjy.forex.trading_trading.controller;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import com.alibaba.fastjson.JSONObject;
import com.sztzjy.forex.trading_trading.annotation.AnonymousAccess;
import com.sztzjy.forex.trading_trading.config.Constant;
import com.sztzjy.forex.trading_trading.entity.ForexMarketData;
import com.sztzjy.forex.trading_trading.entity.Member;
import com.sztzjy.forex.trading_trading.entity.PendingOrder;
import com.sztzjy.forex.trading_trading.entity.TakeStash;
import com.sztzjy.forex.trading_trading.service.*;
import com.sztzjy.forex.trading_trading.util.BigDecimalUtils;
import com.sztzjy.forex.trading_trading.util.ForexMarketDateUtil;
import com.sztzjy.forex.trading_trading.util.RedisUtil;
import com.sztzjy.forex.trading_trading.util.ResultEntity;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

//学生端 交易
@RestController
@RequestMapping("api/trading")
public class TradingController {

    @Autowired
    MemberService memberService;

    @Autowired
    TakeStashService takeStashService;

    @Autowired
    RedisUtil redisUtil;

    @Autowired
    ForexMarketDataService forexMarketDataService;

    @Autowired
    PendingOrderService pendingOrderService;

    @Autowired
    BigDecimalUtils bigDecimalUtils;

    @Autowired
    WainingService wainingService;

    @Autowired
    ForexMarketDateUtil forexMarketDateUtil;


    //获取市场报价
    @AnonymousAccess
    @PostMapping("getMarketQuotation")
    public List<ForexMarketData> getMarketQuotation() {
        boolean bool = redisUtil.hasKey("ForexDateList");
        if (bool == true) {
            List<ForexMarketData> forexDateList = redisUtil.get("ForexDateList");
            return forexDateList;
        }
        List<ForexMarketData> forexDateList = forexMarketDataService.selectLastForexMarketData();
        redisUtil.set("ForexDateList", forexDateList);
        return forexDateList;
    }

    //根据交易类型获取实时数据
    @AnonymousAccess
    @PostMapping("getMarketQuotationByCode")
    public ResultEntity<ForexMarketData> getMarketQuotationByCode(@RequestBody String code) {
        boolean bool = redisUtil.hasKey("ForexDateList");
        if (bool == true) {
            List<ForexMarketData> forexDateList = redisUtil.get("ForexDateList");
            if (forexDateList.size() > 0) {
                for (int i = 0; i < forexDateList.size(); i++) {
                    String getCode = forexDateList.get(i).getCode();
                    if (code.equals(getCode)) {
                        return new ResultEntity(HttpStatus.OK, "获取成功",forexDateList.get(i));
                    }
                }
            }
        }
        List<ForexMarketData> forexDateList = forexMarketDataService.selectLastForexMarketData();
        redisUtil.set("ForexDateList", forexDateList);
        return new ResultEntity(HttpStatus.OK, "获取成功",forexDateList);
    }

    //市场报价交易 交易量为1为1000美元
    @AnonymousAccess
    @PostMapping("transactionMarketQuotation")
    public ResultEntity getMarketQuotation(@RequestBody JSONObject jsonObject) {
        String tradingCode = jsonObject.getString("tradingCode"); //交易品种
        String currencyName = jsonObject.getString("currencyName");
        String transactionType = jsonObject.getString("transactionType"); //交易类型
        Double transactionVolume = jsonObject.getDouble("transactionVolume"); //买卖手（量）
        String buySellType = jsonObject.getString("buySellType");//买入卖出类型
        Double stopLoss = jsonObject.getDouble("stopLoss");
        Double stopWin = jsonObject.getDouble("stopWin");
        String memberId = jsonObject.getString("memberId");
        String trainingId = jsonObject.getString("trainingId");
        Double priceCommission = jsonObject.getDouble("priceCommission"); //获取挂单开仓用户输入的价位
        ForexMarketData forexData = getMarketQuotationByCode(tradingCode).getBody().getData();//获取当前买卖价格
        Double buyPic = forexData.getBuyPic();//当前买价
        Double sellPic = Double.valueOf(forexData.getSellPic());//当前买价
        Double priceTransaction=0.0; //记录当前交易价格(买价或卖价)
        if(priceCommission!=-1){
            buyPic=priceCommission;
            sellPic=priceCommission;
        }
        Member member = memberService.getMemberByMemberIdAndTrainingId(memberId, trainingId);
        Double availableFunds = getAvailableFunds(member); //获取账户可用资金
        Double margin = 0.0; //记录所需保证金
        Boolean stopLossWinFlag=false;
        if (null != stopLoss || null != stopWin) {   //判断止损止赢是否合理 如果stopLoss stopWin都为null则跳过
            stopLossWinFlag=true;
            boolean winOrLossStopBoolean = getWinOrLossStop(stopLoss, stopWin, buySellType, forexData);
            if (winOrLossStopBoolean == false) {
                return new ResultEntity(HttpStatus.BAD_REQUEST, "止损或止盈输入错误");
            }
        }
        //如果方式为卖 则止损高于卖价 获利低于买价
        if (transactionType.equals("sjkc")) {//市价开仓
            String stashId = IdUtil.simpleUUID(); //设置市价开仓ID
            if (tradingCode.startsWith("USD")) { //美元在前
                margin = startUSDMarginNeed(trainingId,memberId,0,tradingCode,buySellType,transactionVolume);
                if(availableFunds<margin){  //判断可用资金是否足够
                    return new ResultEntity(HttpStatus.BAD_REQUEST, "可用资金不足");
                }
                if ("buy".equals(buySellType)) {
                    priceTransaction=buyPic;
                    TakeStash takeStash = returnTakeStash(stashId,memberId, trainingId, tradingCode, currencyName, buySellType, transactionVolume, buyPic, stopLoss, stopWin);
                    takeStashService.insertTakeStash(takeStash);
                } else if ("sell".equals(buySellType)) {
                    priceTransaction=sellPic;
                    TakeStash takeStash = returnTakeStash(stashId,memberId, trainingId, tradingCode, currencyName, buySellType, transactionVolume, sellPic, stopLoss, stopWin);
                    takeStashService.insertTakeStash(takeStash);
                }
                updateMemberAvailableFundsAndMarginUsed(member,margin);
            } else { //美元在后 使用可用资金表达式为 买卖手*1000*卖/买价 判断可用资金是否足够
                margin =endUSDMarginNeed(trainingId,memberId,0,tradingCode,buySellType,transactionVolume,buyPic,sellPic);
                if (availableFunds < margin) {
                    return new ResultEntity(HttpStatus.BAD_REQUEST, "可用资金不足");
                }
                if ("buy".equals(buySellType)) {
                    priceTransaction=buyPic;
                    TakeStash takeStash = returnTakeStash(stashId,memberId, trainingId, tradingCode, currencyName, buySellType, transactionVolume, buyPic, stopLoss, stopWin);
                    takeStashService.insertTakeStash(takeStash);
                } else if ("sell".equals(buySellType)) {
                    priceTransaction=sellPic;
                    TakeStash takeStash = returnTakeStash(stashId,memberId, trainingId, tradingCode, currencyName, buySellType, transactionVolume, sellPic, stopLoss, stopWin);
                    takeStashService.insertTakeStash(takeStash);
                }
                updateMemberAvailableFundsAndMarginUsed(member, margin);
            }
            //保存持仓数据后 保存到redis中
            if(stopLossWinFlag==true){
                Map map=new HashMap();
                if(stopLoss==null){
                    map.put("stopLoss",-1);
                }else {
                    map.put("stopLoss",stopLoss);
                }
                if(stopWin==null){
                    map.put("stopWin",-1);
                }else {
                    map.put("stopWin",stopWin);
                }
                map.put("stashId",stashId);
                map.put("buySellType",buySellType);
                map.put("tradingCode",tradingCode);
                map.put("transactionVolume",transactionVolume);
                map.put("priceTransaction",priceTransaction);
                map.put("memberId",memberId);
                map.put("trainingId_",trainingId);
                redisUtil.hmset("trainingId_"+trainingId+"_stashId_"+stashId,map);
            }
        } else if (transactionType.equals("gdkc")) {//挂单开仓  生成的数据为挂单数据  挂单数据生效 判断可用资金是否充足
            String pendingOrderId = IdUtil.simpleUUID(); //设置市价开仓ID
            Date validityTime = jsonObject.getDate("validityTime"); //获取挂单有效期
            if ("buyLimit".equals(buySellType)) { //限价买进（低价买进）  下单时买入价低于当前买价
                if (priceCommission >= buyPic) {
                    return new ResultEntity(HttpStatus.BAD_REQUEST, "限价买进价位不能高于当前买价");
                }
            }
            if ("sellLimit".equals(buySellType)) {//限价卖出（高价卖出） 下单时卖出价高于当前卖价
                if (priceCommission <= sellPic) {
                    return new ResultEntity(HttpStatus.BAD_REQUEST, "限价卖出价位不能低于当前卖价");
                }
            }
            if ("buyStop".equals(buySellType)) {//止损买进（高价买进） 下单时买入价格高于当前买价
                if (priceCommission <= buyPic) {
                    return new ResultEntity(HttpStatus.BAD_REQUEST, "止损买进价位不能低于当前买价");
                }
            }
            if ("sellStop".equals(buySellType)) {//止损卖出（低价卖出）下单时卖出价格低于当前卖价
                if (priceCommission >= sellPic) {
                    return new ResultEntity(HttpStatus.BAD_REQUEST, "止损卖出价位不能高于当前卖价");
                }
            }
            PendingOrder pendingOrder = returnPendingOrder(pendingOrderId,memberId, trainingId, tradingCode,currencyName,buySellType, transactionVolume, priceCommission, stopLoss, stopWin, validityTime);
            insertPendingOrder(pendingOrder);//保存挂单

            //保存挂单数据后 保存到redis中
            Map map=new HashMap();
            map.put("tradingCode",tradingCode);
            map.put("currencyName",currencyName);
            map.put("transactionType",transactionType);
            map.put("transactionVolume",transactionVolume);
            map.put("buySellType",buySellType);
            map.put("stopLoss",stopLoss);
            map.put("stopWin",stopWin);
            map.put("memberId",memberId);
            map.put("trainingId",trainingId);
            map.put("validityTime",validityTime);
            map.put("pendingOrderId",pendingOrderId);
            map.put("priceCommission",priceCommission);
            redisUtil.hmset("trainingId_"+trainingId+"_pengingOrderId_"+pendingOrderId,map);
        }
        //获取交易次数预警  memberID  tradingID  获取预警次数 判断是否超过 如果超过 则报警
        //checkopeningTrades(memberID,tradingID){}
        return new ResultEntity(HttpStatus.OK, "交易成功");
    }


    //返回持仓对象
    private TakeStash returnTakeStash(String stashId,String memberId, String trainingId, String tradingCode,String currencyName, String buySellType, Double transactionVolume, Double buyOrSellPic, Double stopLoss, Double stopWin) {
        TakeStash takeStash = new TakeStash();
        Date now = new Date();
        String orderNumberStr = DateUtil.format(now, "yyyyMMddHHmmss") + System.currentTimeMillis();
        takeStash.setStashId(stashId);
        takeStash.setMemberId(memberId);
        takeStash.setTrainingId(trainingId);
        takeStash.setTradingCode(tradingCode);
        takeStash.setCurrencyName(currencyName);
        takeStash.setOrderNumber(orderNumberStr);  //根据时间生成
        takeStash.setBuySellType(buySellType);
        takeStash.setVolumeTransaction(transactionVolume);
        takeStash.setPriceTransaction(buyOrSellPic);
        takeStash.setStopLoss(stopLoss);
        takeStash.setStopWin(stopWin);
        takeStash.setTimeTransaction(now);
        takeStash.setStatus(0);
        return takeStash;
    }

    //更新已用保证金
    private void updateMemberAvailableFundsAndMarginUsed(Member member,Double margin) {
        Double marginUsed=member.getMarginUsed();
        if (marginUsed==null){
            marginUsed=0.0;
        }
        member.setMarginUsed(bigDecimalUtils.mul(bigDecimalUtils.add(marginUsed,margin),1,2)); //设置已用保证金
        Integer openingTrades = member.getOpeningTrades(); //获取开仓次数
        if(openingTrades==null){
            openingTrades=0;
        }
        member.setOpeningTrades(openingTrades++);//设置开仓次数
        memberService.updateByPrimaryKeySelective(member);
        wainingService.compareMarginLevels(member.getMemberId(),member.getTrainingId());//更改可用保证金后 调用预警
    }

    //判断止损止赢  逻辑：如果方式为买 则止损低于买价 获利高于买价 / 如果方式为卖 则止损高于卖价 获利低于买价 （可以只传一个参数）
    public boolean getWinOrLossStop(Double stopLoss, Double stopWin, String buySellType, ForexMarketData forexData) {
        if ("buy".equals(buySellType)) {
            Double buyPic = forexData.getBuyPic();
            //判断stopLoss 和stopWin 是否为空
            if (null != stopLoss && null != stopWin) {
                if (stopLoss < buyPic && stopWin > buyPic) {
                    return true;
                }
                return false;
            }
            if (null == stopLoss) {
                if (stopWin > buyPic) {
                    return true;
                }
                return false;
            }
            if (null == stopWin) {
                if (stopLoss < buyPic) {
                    return true;
                }
                return false;
            }

        } else if ("sell".equals(buySellType)) {
            Double sellPic = Double.valueOf(forexData.getSellPic());
            if (null != stopLoss && null != stopWin) {
                if (stopLoss > sellPic && stopWin < sellPic) {
                    return true;
                }
                return false;
            }
            if (null == stopLoss) {
                if (stopWin < sellPic) {
                    return true;
                }
                return false;
            }
            if (null == stopWin) {
                if (stopLoss > sellPic) {
                    return true;
                }
                return false;
            }
        }
        return false;
    }

    //返回挂单对象
    public PendingOrder returnPendingOrder(String pendingOrderId,String memberId, String trainingId, String tradingCode,String currencyName, String buySellType, Double transactionVolume, Double priceCommission, Double stopLoss, Double stopWin,Date validityTime) {
        Date now = new Date();
        String commissionNumber = DateUtil.format(now, "yyyyMMddHHmmss") + System.currentTimeMillis();
        PendingOrder pendingOrder=new PendingOrder();
        pendingOrder.setPendingOrderId(pendingOrderId);
        pendingOrder.setMemberId(memberId);
        pendingOrder.setTrainingId(trainingId);
        pendingOrder.setCommissionTime(now);
        pendingOrder.setTradingCode(tradingCode);
        pendingOrder.setCurrencyName(currencyName);
        pendingOrder.setCommissionNumber(commissionNumber);
        pendingOrder.setBuySellType(buySellType);
        pendingOrder.setVolumeTransaction(transactionVolume);
        pendingOrder.setPriceCommission(priceCommission);
        pendingOrder.setStopLoss(stopLoss);
        pendingOrder.setStopWin(stopWin);
        pendingOrder.setValidityTime(validityTime);
        pendingOrder.setStatus(0);
        return pendingOrder;
    }

    //保存挂单  挂单保存后 将挂单ID和有效期存入redis
    public void insertPendingOrder(PendingOrder pendingOrder){
        String uuid = IdUtil.simpleUUID();
        pendingOrder.setPendingOrderId(uuid);
        pendingOrderService.insert(pendingOrder);
    }

    //美元在前  返回保证金变化量
    public Double startUSDMarginNeed(String trainingId,String memberId,int status,String tradingCode,String buySellType,Double transactionVolume){
        Double margin=0.0;
        //获取持仓表中tradingCode所有buysell情况  根据trainingId和memberId查询交易品种=tradingCode的所有数据
        List<TakeStash> takeStashList=takeStashService.findTakeStashByTrainingIdAndMemberIdAndStatusAndTradingCode(trainingId,memberId,status,tradingCode);
        Double buyTotalVolumeTransaction=0.0;
        Double sellTotalVolumeTransaction=0.0;
        if(takeStashList.isEmpty()){
            margin=transactionVolume*Constant.LEVERQUANTITY;
            return margin;
        }else {
            for (int i = 0; i < takeStashList.size(); i++) {
                TakeStash takeStash = takeStashList.get(i);
                String buySellTypeTakeStash = takeStash.getBuySellType();
                if("buy".equals(buySellTypeTakeStash)){
                    buyTotalVolumeTransaction=buyTotalVolumeTransaction+takeStash.getVolumeTransaction();
                }else {
                    sellTotalVolumeTransaction=sellTotalVolumeTransaction+takeStash.getVolumeTransaction();
                }
            }
        }
        if("buy".equals(buySellType)){
            if(sellTotalVolumeTransaction>buyTotalVolumeTransaction){
                if(sellTotalVolumeTransaction>buyTotalVolumeTransaction+transactionVolume){
                    margin=0.0;
                }else {
                    margin=(transactionVolume+buyTotalVolumeTransaction-sellTotalVolumeTransaction)*Constant.LEVERQUANTITY;
                }
            }else {
                margin=transactionVolume*Constant.LEVERQUANTITY;
            }
        }else {
            if(buyTotalVolumeTransaction>sellTotalVolumeTransaction){
                if(buyTotalVolumeTransaction>sellTotalVolumeTransaction+transactionVolume){
                    margin=0.0;
                }else {
                    margin=(transactionVolume+sellTotalVolumeTransaction-buyTotalVolumeTransaction)*Constant.LEVERQUANTITY;
                }
            }else {
                margin=transactionVolume*Constant.LEVERQUANTITY;
            }
        }
        return margin;
    }

    //美元在后  返回保证金变化量  公式（（sell交易量*交易价格+sell交易量*价格+...）/交易总量+(buy交易量*交易价格+buy交易量*价格+...)/交易总量）*合约量/杠杆倍数*sellbuy最高交易量
    public Double endUSDMarginNeed(String trainingId,String memberId,int status,String tradingCode,String buySellType,Double transactionVolume,Double buyPic,Double sellPic){
        Double margin=0.0;
        //获取持仓表中tradingCode所有buysell情况  根据trainingId和memberId查询交易品种=tradingCode的所有数据
        List<TakeStash> takeStashList=takeStashService.findTakeStashByTrainingIdAndMemberIdAndStatusAndTradingCode(trainingId,memberId,status,tradingCode);
        Double buyTotalMargin=0.0;  //类型为买的 交易量*交易价格之和
        Double sellTotalMargin=0.0; //类型为卖的 交易量*交易价格之和
        Double totalVolumeTransaction=0.0; //交易总量
        Double buyTotalVolumeTransaction=0.0;  //buy类型交易总量
        Double sellTotalVolumeTransaction=0.0; //sell类型交易总量
        Double highVolumeTransaction=0.0; //记录buy/sell类型交易总量中较高的一个
        if(takeStashList.isEmpty()){
            if("buy".equals(buySellType)){
                margin=transactionVolume*buyPic*Constant.LEVERQUANTITY;
            }else {
                margin=transactionVolume*sellPic*Constant.LEVERQUANTITY;
            }
            return margin;
        }else {
            for (int i = 0; i < takeStashList.size(); i++) {
                TakeStash takeStash = takeStashList.get(i);
                String buySellTypeTakeStash = takeStash.getBuySellType();
                Double volumeTransactionTakeStash = takeStash.getVolumeTransaction(); //持仓交易量
                Double priceTransactionTakeStash = takeStash.getPriceTransaction(); //持仓交易价格
                if("buy".equals(buySellTypeTakeStash)){
                    buyTotalMargin=buyTotalMargin+volumeTransactionTakeStash*priceTransactionTakeStash;
                    buyTotalVolumeTransaction=buyTotalVolumeTransaction+volumeTransactionTakeStash;
                }else {
                    sellTotalMargin=sellTotalMargin+volumeTransactionTakeStash*priceTransactionTakeStash;
                    sellTotalVolumeTransaction=sellTotalVolumeTransaction+volumeTransactionTakeStash;
                }
                totalVolumeTransaction=totalVolumeTransaction+volumeTransactionTakeStash;
            }
        }
        if (buyTotalVolumeTransaction>=sellTotalVolumeTransaction){  //设置带sell\buy的最高交易交易总量
            highVolumeTransaction=buyTotalVolumeTransaction;
        }else {
            highVolumeTransaction=sellTotalVolumeTransaction;
        }
        Double marginBeforeTrading=(buyTotalMargin+sellTotalMargin)/totalVolumeTransaction*Constant.LEVERQUANTITY*highVolumeTransaction; //进仓之前的保证金
        totalVolumeTransaction=totalVolumeTransaction+transactionVolume;  //交易总量增加本次进仓的交易量
        if("buy".equals(buySellType)){
            buyTotalMargin=buyTotalMargin+transactionVolume*buyPic;
            if (buyTotalVolumeTransaction+transactionVolume>=sellTotalVolumeTransaction){  //设置带sell\buy的最高交易交易总量
                highVolumeTransaction=buyTotalVolumeTransaction+transactionVolume;
            }else {
                highVolumeTransaction=sellTotalVolumeTransaction;
            }
        }else {
            sellTotalMargin=sellTotalMargin+transactionVolume*sellPic;
            if (sellTotalVolumeTransaction+transactionVolume>=buyTotalVolumeTransaction){  //设置带sell\buy的最高交易交易总量
                highVolumeTransaction=sellTotalVolumeTransaction+transactionVolume;
            }else {
                highVolumeTransaction=buyTotalVolumeTransaction;
            }
        }
        Double marginAfterTrading=(buyTotalMargin+sellTotalMargin)/totalVolumeTransaction*Constant.LEVERQUANTITY*highVolumeTransaction;
        margin=marginAfterTrading-marginBeforeTrading;
        return margin;
    }

    //获取可用资金
    public Double getAvailableFunds(Member member){
        Double positionProfitLoss = flashTotalPositionProfitLoss(member.getMemberId());
        if (positionProfitLoss==null){
            positionProfitLoss=0.0;
        }
        Double initialCapital = member.getInitialCapital();//初始资金
        Double totalAssets = initialCapital+positionProfitLoss;
        Double netValue = bigDecimalUtils.add(totalAssets, positionProfitLoss);
        Double cumulativeProfitLoss = member.getCumulativeProfitLoss(); //累计盈亏
        if(cumulativeProfitLoss==null){
            member.setCumulativeProfitLoss(0.0);
        }
        Double marginUsed = member.getMarginUsed(); //获取已用保证金
        if (marginUsed==null){
            marginUsed=0.0;
        }
        return bigDecimalUtils.sub(netValue, marginUsed);
    }

    public Double flashTotalPositionProfitLoss(String memberId) {
        List<TakeStash> takeStashList = takeStashService.selectAllByMemberIdAndStatus(memberId, 0);
        Double totalProfitAndLoss = 0.0;
        for (int i = 0; i < takeStashList.size(); i++) {
            TakeStash takeStash = takeStashList.get(i);
            String buySellType = takeStash.getBuySellType();  //买入或卖出
            String tradingCode = takeStash.getTradingCode();  //交易品种
            Double priceTransaction = takeStash.getPriceTransaction();  //交易价格
            Double volumeTransaction = takeStash.getVolumeTransaction(); //交易量
            ForexMarketData forexData = forexMarketDateUtil.getForexMarketDateByCode(tradingCode);
            Double nowBuyPic = forexData.getBuyPic();  //当前买价
            Double nowSellPic = Double.valueOf(forexData.getSellPic());  //当前卖价
            Double profitAndLoss;
            if (tradingCode.startsWith("USD")) { //美元在前
                if ("buy".equals(buySellType)) { //买
                    profitAndLoss = (nowSellPic - priceTransaction) * volumeTransaction * Constant.LEVERQUANTITY / nowSellPic;
                } else { //卖
                    profitAndLoss = (priceTransaction - nowBuyPic) * volumeTransaction * Constant.LEVERQUANTITY / nowBuyPic;
                }
            } else {  //美元在后
                if ("buy".equals(buySellType)) { //买
                    profitAndLoss = (nowSellPic - priceTransaction) * volumeTransaction * Constant.LEVERQUANTITY;
                } else { //卖
                    profitAndLoss = (nowBuyPic - priceTransaction) * volumeTransaction * Constant.LEVERQUANTITY;
                }
            }
            totalProfitAndLoss = totalProfitAndLoss + profitAndLoss;
            totalProfitAndLoss = bigDecimalUtils.mul(totalProfitAndLoss, 1, 2);
        }
        return totalProfitAndLoss;
    }

}
