package com.sztzjy.forex.trading_trading.controller;


import cn.hutool.core.util.IdUtil;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageInfo;
import com.sztzjy.forex.trading_trading.annotation.Permission;
import com.sztzjy.forex.trading_trading.annotation.aspect.PermissionType;
import com.sztzjy.forex.trading_trading.config.security.JwtUser;
import com.sztzjy.forex.trading_trading.config.security.TokenProvider;
import com.sztzjy.forex.trading_trading.dto.TrainingBO;
import com.sztzjy.forex.trading_trading.entity.GradeWeight;
import com.sztzjy.forex.trading_trading.entity.Member;
import com.sztzjy.forex.trading_trading.entity.Training;
import com.sztzjy.forex.trading_trading.service.GradeWeightService;
import com.sztzjy.forex.trading_trading.service.MemberService;
import com.sztzjy.forex.trading_trading.service.TrainingService;
import com.sztzjy.forex.trading_trading.util.ResultEntity;
import com.sztzjy.forex.trading_trading.util.TzApi;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;
import org.w3c.dom.stylesheets.LinkStyle;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.*;
import java.util.stream.Collectors;

@Api(tags = "实训管理")
@RestController
@RequestMapping("api/training")
@RequiredArgsConstructor
public class TrainingController {

    private final TrainingService trainingService;
    private final MemberService memberService;
    private final HttpServletRequest request;

    private final GradeWeightService gradeWeightService;


    @Permission(codes = PermissionType.TRAINING_MANAGEMENT_ADD)
    @ApiOperation("教师端----新增一条实训记录")
    @PostMapping("create")
    public ResultEntity create(@RequestBody TrainingBO bo) {
        Assert.notNull(bo.getClassId(), "请选择班级");
        JwtUser currentUser = TokenProvider.getJWTUser(request);

        Training training = trainingService.create(bo, currentUser);
        String classIds = bo.getClassId().stream()
                .map(String::valueOf)
                .collect(Collectors.joining(","));
        List<Map<String, Object>> studentInfos = TzApi.GetStudentInfoByClassIdForForeignExchangeTrading(classIds, currentUser);
        if (bo.getAddUserInfos() != null && bo.getAddUserInfos().size() > 0) {
            studentInfos.addAll(bo.getAddUserInfos());
        }
        List<Member> members = buildMembers(studentInfos, training);
        if (members != null && members.size() > 0) memberService.insertAll(members);
        training.setPeopleCount(members.size());
        GradeWeight gradeWeight = gradeWeightService.findBySchoolId(currentUser.getSchoolId()) == null
                ? gradeWeightService.createDefaultWeight(currentUser)
                : gradeWeightService.findBySchoolId(currentUser.getSchoolId());
        training.setWeightId(gradeWeight.getWeightId());
        trainingService.update(training);

        //生成训练账号（member）
        buildPracticeMembers(members);

        return new ResultEntity(HttpStatus.OK);
    }


    @Permission(codes = PermissionType.TRAINING_MANAGEMENT_DEL)
    @ApiOperation("教师端----删除指定实训记录")
    @PostMapping("deleteById")
    public ResultEntity deleteById(@ApiParam("实训记录id") @RequestParam String trainingId) {
        trainingService.delete(trainingId);
        //TODO 同步删除member表数据
//        memberService
        return new ResultEntity(HttpStatus.OK);
    }

    @Permission(codes = PermissionType.TRAINING_MANAGEMENT_EDIT)
    @ApiOperation("教师端----修改一条实训记录")
    @PostMapping("update")
    public ResultEntity update(@ApiParam("实训记录id") @RequestParam String trainingId,
                               @ApiParam("实训记录名字") @RequestParam String trainingName) {

        //TODO 暂时只修改实训名字 待后续需求
        trainingService.update(trainingId, trainingName);
        return new ResultEntity(HttpStatus.OK);
    }


    @Permission(codes = PermissionType.TRAINING_MANAGEMENT_SEARCH)
    @ApiOperation("教师端----根据实训记录id查询实训记录")
    @GetMapping
    public ResultEntity<Training> get(@ApiParam("实训记录id")
                                      @PathVariable String id) {
        //TODO 后续还有根据实训id查看排行榜等需求待实现
        return new ResultEntity<Training>(HttpStatus.OK, trainingService.findById(id));
    }


    @Permission(codes = PermissionType.TRAINING_MANAGEMENT_SEARCH)
    @ApiOperation("教师端----根据条件分页获取实训记录数据")
    @GetMapping("findByConditions")
    public ResultEntity<PageInfo<Training>> findByConditions(@ApiParam("页码") Integer index,
                                                             @ApiParam("页量") Integer size) {
        //TODO 待确认需过滤参数
        JwtUser currentUser = TokenProvider.getJWTUser(request);
        return new ResultEntity<PageInfo<Training>>(HttpStatus.OK, trainingService.pagedListTraining(currentUser.getSchoolId(), index, size));
    }

    @Permission(codes = PermissionType.TRAINING_MANAGEMENT_SEARCH)
    @ApiOperation("获取实训下拉框列表")
    @GetMapping("findTrainingNameList")
    public ResultEntity<List<Map<String, Object>>> findTrainingNameList() {
        JwtUser currentUser = TokenProvider.getJWTUser(request);
        return new ResultEntity<List<Map<String, Object>>>(HttpStatus.OK, trainingService.findTrainingNameList(currentUser));
    }

    @Permission(codes = PermissionType.TRAINING_MANAGEMENT_ADD)
    @ApiOperation("教师端----添加实训成员时用：获取班级列表")
    @GetMapping("findClassNameList")
    public ResultEntity<List<Map<String, Object>>> findClassNameList(@ApiParam("专业Id") Integer majorId) {
        JwtUser currentUser = TokenProvider.getJWTUser(request);
        List<Map<String, Object>> mapList;
        if (majorId == null || majorId == 0) {
            mapList = TzApi.GetClassBySchoolIdForForeignExchangeTrading(currentUser);
        } else {
            mapList = TzApi.GetClassByMajorIdForForeignExchangeTrading(currentUser, majorId);
        }
        return new ResultEntity<List<Map<String, Object>>>(HttpStatus.OK, mapList);
    }

    @Permission(codes = PermissionType.TRAINING_MANAGEMENT_ADD)
    @ApiOperation("教师端----根据所选的班级获取学生列表（分页展示）")
    @GetMapping("findStudentListByClassIds")
    public ResultEntity<PageInfo<Map<String, Object>>> findStudentListByClassIds(@ApiParam("页码") Integer index,
                                                                                 @ApiParam("页量") Integer size,
                                                                                 @ApiParam("班级id列表") @RequestParam List<Integer> classIds) {
        JwtUser currentUser = TokenProvider.getJWTUser(request);
        return new ResultEntity<PageInfo<Map<String, Object>>>(HttpStatus.OK, TzApi.PagedListStudentInfoByClassIdForForeignExchangeTrading(
                currentUser,
                classIds.stream()
                        .map(String::valueOf)
                        .collect(Collectors.joining(",")),
                index,
                size));
    }

    @Permission(codes = PermissionType.TRAINING_MANAGEMENT_ADD)
    @ApiOperation("教师端----添加实训成员时用：根据学号或者学生名字获取用户信息")
    @GetMapping("findStudentInfoByKeyword")
    public ResultEntity<List<Map<String, Object>>> findStudentInfoByKeyword(@ApiParam("关键字") @RequestParam String keyword) {
        JwtUser currentUser = TokenProvider.getJWTUser(request);
        return new ResultEntity<List<Map<String, Object>>>(HttpStatus.OK, TzApi.GetStudentInfoByKeywordForForeignExchangeTrading(currentUser, keyword));
    }

    @Permission(codes = PermissionType.TRAINING_MANAGEMENT_ADD)
    @ApiOperation("教师端----添加实训成员时用：获取专业列表下拉框")
    @GetMapping("findMajorNameList")
    public ResultEntity<List<Map<String, Object>>> findMajorNameList() {
        JwtUser currentUser = TokenProvider.getJWTUser(request);
        return new ResultEntity<List<Map<String, Object>>>(HttpStatus.OK, TzApi.GetMajorIdForForeignExchangeTrading(currentUser));
    }

    @Permission(codes = PermissionType.TRAINING_MANAGEMENT_EDIT)
    @ApiOperation("教师端----成员管理：在已存在的实训记录添加参赛人员")
    @PostMapping("addMember")
    public ResultEntity addMember(@ApiParam("实训记录id") @RequestParam String trainingId,
                                  @ApiParam("查询关键字：用户名字或学号") @RequestParam String keyword) {
        Assert.notNull(trainingId, "实训记录不存在");
        Assert.isTrue(StringUtils.hasText(keyword), "查询关键字不能为空");

        JwtUser currentUser = TokenProvider.getJWTUser(request);
        Training training = trainingService.findById(trainingId);
        Assert.notNull(training, "实训记录不存在");
        List<Map<String, Object>> maps = TzApi.GetStudentInfoByKeywordForForeignExchangeTrading(currentUser, keyword);
        if (maps == null || maps.size() == 0) {
            throw new NullPointerException("关键字未查询到用户");
        }
        buildMembers(maps, training).forEach(member -> {
            memberService.insert(member);
            buildPracticeMember(member);
        });
        trainingService.updatePeopleNum(true, trainingId);
        return new ResultEntity(HttpStatus.OK);
    }


    @Permission(codes = PermissionType.TRAINING_MANAGEMENT_DEL)
    @ApiOperation("教师端----成员管理：删除指定大赛的参赛用户")
    @PostMapping("deleteMember")
    public ResultEntity deleteMember(String trainingId, String memberId) {
        Assert.notNull(trainingId, "实训记录不存在");
        Assert.notNull(memberId, "参赛用户不存在");
        memberService.deleteById(memberId);
        trainingService.updatePeopleNum(false, trainingId);
        return new ResultEntity(HttpStatus.OK);
    }


    private List<Member> buildMembers(List<Map<String, Object>> list, Training training) {
        List<Member> members = new ArrayList<>();
        Set<String> userNos = new HashSet<>();
        if (list != null && list.size() > 0) {
            for (Map<String, Object> map : list) {
                if (!userNos.add(String.valueOf(map.get("studentNo")))) {
                    continue;
                }
                Member member = new Member();
                member.setMemberId(IdUtil.simpleUUID());
                member.setName(String.valueOf(map.get("name")));
                member.setTrainingId(training.getTrainingId());
                member.setSchoolId(training.getSchoolId());
                member.setClassGrade(String.valueOf(map.get("className")));
                member.setClassId(Integer.valueOf(map.get("classId").toString()));
                member.setMajor(String.valueOf(map.get("majorName")));
                member.setStudentNumber(String.valueOf(map.get("studentNo")));
                member.setTrainingName(training.getTrainingName());
                member.setInitialCapital(1000000.0);
                member.setMarginUsed(0.0);
                member.setCumulativeProfitLoss(0.0);
                member.setOpeningTrades(0);
                member.setClosingTrades(0);
                members.add(member);
            }

        }
        return members;
    }

    private void buildPracticeMembers(List<Member> members) {
        for (int i = 0; i < members.size(); i++) {
            Member member = members.get(i);
            member.setMemberId(IdUtil.simpleUUID());
            member.setTrainingId("999999999");
            member.setTrainingName("AdminStudentPractice");
            member.setSchoolId(999999999);
        }
        memberService.insertAll(members);
    }

    private void buildPracticeMember(Member member) {
        member.setMemberId(IdUtil.simpleUUID());
        member.setTrainingId("999999999");
        member.setTrainingName("AdminStudentPractice");
        member.setSchoolId(999999999);
        memberService.insert(member);
    }


    @Permission(codes = PermissionType.TRAINING_MANAGEMENT_SEARCH)
    @ApiOperation("获取我的任务列表")
    @GetMapping("/findTrainingList")
//    @AnonymousAccess
    public ResultEntity<PageInfo<Training>> findTrainingList(@ApiParam("页码") @RequestParam Integer index,
                                                             @ApiParam("页量") @RequestParam Integer size) {
        JwtUser currentUser = TokenProvider.getJWTUser(request);
        Integer schoolId = currentUser.getSchoolId();
        String username = currentUser.getUsername();
        return new ResultEntity<PageInfo<Training>>(HttpStatus.OK, trainingService.findTrainingList(index, size, schoolId, username));
    }

    @Permission(codes = PermissionType.TRAINING_MANAGEMENT_SEARCH)
    @ApiOperation("根据实训记录id获取班级列表")
    @GetMapping("/findClassListByTrainingId")
    public ResultEntity<List<Map<String, Object>>> findClassListByTrainingId(@ApiParam("实训记录id") @RequestParam String trainingId) {
        Assert.hasText(trainingId, "实训记录id不能为空");
        return new ResultEntity<List<Map<String, Object>>>(HttpStatus.OK, memberService.selectClassListByTrainingId(trainingId));
    }

    @Permission(codes = PermissionType.TRAINING_MANAGEMENT_SEARCH)
    @ApiOperation("根据班级id获取实训列表")
    @GetMapping("/findTrainingListByClassId")
    public ResultEntity<List<Map<String, Object>>> findTrainingListByClassId(@ApiParam("实训记录id") String classId) {
        if (!StringUtils.hasText(classId)) {
            JwtUser user = TokenProvider.getJWTUser(request);
            return new ResultEntity<>(HttpStatus.OK, memberService.selectAllTrainingList(user.getSchoolId()));
        }
        return new ResultEntity<>(HttpStatus.OK, memberService.selectTrainingListByClassId(classId));
    }

    @Permission(codes = PermissionType.TRAINING_MANAGEMENT_EDIT)
    @Transactional(rollbackFor = Exception.class)
    @ApiOperation("更新参赛人员")
    @GetMapping("/updateMember")
    public ResultEntity updateMember(@ApiParam("实训记录id") @RequestParam String trainingId) {
        JwtUser user = TokenProvider.getJWTUser(request);
        List<Map<String, Object>> classList = memberService.selectClassListByTrainingId(trainingId);
        List<String> insertNames = new ArrayList<>();

        if (classList == null || classList.size() == 0) {
            return new ResultEntity(HttpStatus.OK);
        }
        List<Map<String, Object>> resultInfos = new ArrayList<>();
        List<String> studentNos = new ArrayList<>();
        String classIdResult = classList.stream()
                .map(classMap -> classMap.get("class_id").toString())
                .collect(Collectors.joining(","));
        List<Map<String, Object>> studentInfos = TzApi.GetStudentInfoByClassIdForForeignExchangeTrading(classIdResult, user);
        Training training = trainingService.findById(trainingId);
        for (Map<String, Object> studentInfo : studentInfos) {
            String studentNo = studentInfo.get("studentNo").toString();
            studentNos.add(studentNo);
            Member existsInMember = memberService.findByTrainingIdAndStudentNumber(trainingId,studentNo);
            if (existsInMember!=null) continue;
            insertNames.add(studentInfo.get("name").toString());
            resultInfos.add(studentInfo);
        }
        if(resultInfos==null||resultInfos.size()==0){
            return new ResultEntity(HttpStatus.OK,new ArrayList<>());
        }
        memberService.insertAll(buildMembers(resultInfos, training));
        memberService.deleteMembersByTrainingIdNotInStudentInfos(trainingId,studentNos);
        return new ResultEntity(HttpStatus.OK, insertNames);
    }
}
