package com.sztzjy.forex.trading_trading.controller;


import com.github.pagehelper.PageInfo;
import com.sztzjy.forex.trading_trading.annotation.AnonymousAccess;
import com.sztzjy.forex.trading_trading.annotation.Permission;
import com.sztzjy.forex.trading_trading.annotation.aspect.PermissionType;
import com.sztzjy.forex.trading_trading.config.security.JwtUser;
import com.sztzjy.forex.trading_trading.config.security.TokenProvider;
import com.sztzjy.forex.trading_trading.entity.TrainingLessonPlan;
import com.sztzjy.forex.trading_trading.service.TrainingLessonPlanService;
import com.sztzjy.forex.trading_trading.util.ResultEntity;
import com.sztzjy.forex.trading_trading.util.file.IFileUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import org.apache.http.util.Asserts;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

@Api(tags = "实训教案管理")
@RestController
@RequestMapping("api/trainingLessonPlan")
@RequiredArgsConstructor
public class TrainingLessonPlanController {

    private final TrainingLessonPlanService trainingLessonPlanService;
    private final HttpServletRequest request;
    private final IFileUtil fileUtil;

    @Permission(codes = PermissionType.TRAINING_PLAN_MANAGEMENT_ADD)
    @ApiOperation("教师端----新增一条实训教案记录")
    @PostMapping("create")
    public ResultEntity create(@ApiParam("教案名称") @RequestParam String name,
                               @RequestPart("file") MultipartFile file) {
        JwtUser currentUser = TokenProvider.getJWTUser(request);
        Asserts.notNull(name, "教案名称不能为空");
        Asserts.notNull(file, "请上传文件");
        trainingLessonPlanService.create(name, file, currentUser);
        return new ResultEntity(HttpStatus.OK);
    }

    @Permission(codes = PermissionType.TRAINING_PLAN_MANAGEMENT_DEL)
    @ApiOperation("教师端----删除一条实训教案记录")
    @DeleteMapping("delete")
    public ResultEntity delete(@ApiParam("教案id") @RequestParam String id) {
        Asserts.notNull(id, "教案id不能为空");
        trainingLessonPlanService.deleteById(id);
        return new ResultEntity(HttpStatus.OK);
    }

    @AnonymousAccess
    @ApiOperation("文件下载")
    @GetMapping("download")
    public void download(String id, String TOKEN,HttpServletResponse response) {
        TokenProvider.getJWTUser(TOKEN);
        TrainingLessonPlan trainingLessonPlan = trainingLessonPlanService.findById(id);
        Asserts.notNull(trainingLessonPlan, "教案不存在");
        fileUtil.download(response, trainingLessonPlan.getFilePath());
    }

    @Permission(codes = PermissionType.TRAINING_PLAN_MANAGEMENT_SEARCH)
    @ApiOperation("根据教案名字查询教案(分页展示)")
    @GetMapping("findByConditions")
    public ResultEntity<PageInfo<TrainingLessonPlan>> findByConditions(@ApiParam("教案名称")  String name,
                                             @ApiParam("分页索引：{0}为第一页") @RequestParam(required = false) Integer index,
                                             @ApiParam("页量") @RequestParam(required = false) Integer size) {
        JwtUser currentUser = TokenProvider.getJWTUser(request);
        return new ResultEntity(HttpStatus.OK, trainingLessonPlanService.findByConditions(name,currentUser,index,size));
    }

}
