package com.sztzjy.forex.trading_trading.controller;


import com.sztzjy.forex.trading_trading.annotation.AnonymousAccess;
import com.sztzjy.forex.trading_trading.annotation.OperateLog;
import com.sztzjy.forex.trading_trading.config.security.JwtUser;
import com.sztzjy.forex.trading_trading.config.security.LoginResult;
import com.sztzjy.forex.trading_trading.config.security.TokenProvider;
import com.sztzjy.forex.trading_trading.entity.RoleRoute;
import com.sztzjy.forex.trading_trading.service.RoleAuthorityService;
import com.sztzjy.forex.trading_trading.service.RoleRouteService;
import com.sztzjy.forex.trading_trading.util.ResultEntity;
import com.sztzjy.forex.trading_trading.util.RsaUtil;
import com.sztzjy.forex.trading_trading.util.TzApi;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.session.SessionInformation;
import org.springframework.security.core.session.SessionRegistry;
import org.springframework.security.web.authentication.logout.SecurityContextLogoutHandler;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

@Api(tags = "用户管理模块")
@RestController
@RequestMapping("api/sysUser")
@RequiredArgsConstructor
public class UserController {


    private final RoleAuthorityService roleAuthorityService;

    private final SessionRegistry sessionRegistry;
    private final RoleRouteService roleRouteService;

    private final HttpServletRequest request;

    @AnonymousAccess
    @OperateLog(description = "用户登录")
    @ApiOperation(value = "用户登录", httpMethod = "POST")
    @PostMapping("login")
    public ResultEntity<LoginResult> login(@ApiParam("用户名") @RequestParam String username,
                                           @ApiParam("加密后的密文") @RequestParam String passwordEncode) {

        String password;
        try {
            password = RsaUtil.decryptByPrivateKey(passwordEncode);
        } catch (Exception e) {
            throw new IllegalArgumentException("密码错误");
        }
        String md5Pwd = RsaUtil.calculateMD5(password);
        String hashPwd = RsaUtil.formatHash(md5Pwd);
        JwtUser jwtUser = TzApi.foreignExchangeTradingLogin(username, hashPwd);
        jwtUser.setAuthorityCodes(roleAuthorityService.getAuthorityByRoleId(jwtUser.getRoleId()));
        Authentication authentication = new UsernamePasswordAuthenticationToken(jwtUser, null, jwtUser.getAuthorities());
        SecurityContextHolder.getContext().setAuthentication(authentication);
        String token = TokenProvider.createToken(jwtUser);
        return new ResultEntity<LoginResult>(LoginResult.create(jwtUser, token));
    }


    @AnonymousAccess
    @OperateLog(description = "通过智云平台token单点登录")
    @ApiOperation(value = "通过智云平台token单点登录", httpMethod = "POST")
    @PostMapping("loginByZhiYunToken")
    public ResultEntity<LoginResult> loginByZhiYunToken(@ApiParam("智云平台token") @RequestParam String zhiYunToken) {
        Assert.isTrue(StringUtils.hasText(zhiYunToken), "token不能为空");
        JwtUser jwtUser = TokenProvider.getJWTUserByZhiYun(zhiYunToken);
        Authentication authentication = new UsernamePasswordAuthenticationToken(jwtUser, null, jwtUser.getAuthorities());
        SecurityContextHolder.getContext().setAuthentication(authentication);
        String token = TokenProvider.createToken(jwtUser);
        return new ResultEntity<LoginResult>(LoginResult.create(jwtUser, token));
    }



    @ApiOperation(value = "注销登录", httpMethod = "POST")
    @PostMapping("/logout")
    public ResultEntity logout(HttpServletRequest request) {
        // 获取当前认证信息
        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        JwtUser user = (JwtUser) authentication.getPrincipal();
        if (authentication != null) {
            // 执行注销操作
            new SecurityContextLogoutHandler().logout(request, null, authentication);
            sessionRegistry.removeSessionInformation(user.getUsername());
        }
        return new ResultEntity(HttpStatus.OK);
    }



//    @Permission()
    @AnonymousAccess
    @ApiOperation("获取当前在线用户")
    @GetMapping("online-users")
    public Set<String> getOnlineUsers() {
        List<Object> principals = sessionRegistry.getAllPrincipals();
        Set<String> usernames = new HashSet<>();

        for (Object principal : principals) {
            if (principal instanceof JwtUser) {
                JwtUser user = (JwtUser) principal;
                List<SessionInformation> sessions = sessionRegistry.getAllSessions(user, false);
                if (!sessions.isEmpty()) {
                    usernames.add(user.getUsername());
                }
            }
        }
        return usernames;
    }


    @ApiOperation("根据用户角色获取可访问路由")
    @GetMapping("getRoutesByRole")
    public ResultEntity<List<String>> getRoutesByRole() {
        JwtUser current = TokenProvider.getJWTUser(request);
        return new ResultEntity<List<String>>(roleRouteService.findRoutesByRoleId(current.getRoleId()));
    }
}



