package com.sztzjy.forex.trading_trading.controller;


import cn.hutool.core.util.IdUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.sztzjy.forex.trading_trading.entity.*;
import com.sztzjy.forex.trading_trading.mappers.*;
import com.sztzjy.forex.trading_trading.service.WainingService;
import com.sztzjy.forex.trading_trading.util.ResultEntity;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@Api(tags = "预警页面")
@RestController
@RequestMapping("api/warning")
public class WarningController {
    @Autowired
    private WainingService wainingService;
    @Autowired
    private MarketWarningMapper marketWarningMapper;
    @Autowired
    private MarketWarningRecordMapper marketWarningRecordMapper;
    @Autowired
    private AlertHoldDurationMapper alertHoldDurationMapper;
    @Autowired
    private AlertHoldDurationRecordMapper alertHoldDurationRecordMapper;
    @Autowired
    private TransactionNumberWarningMapper transactionNumberWarningMapper;
    @Autowired
    private TransactionNumberWarningRecordMapper transactionNumberWarningRecordMapper;

    //保证金预警页面展示
    @GetMapping("/getMarginWarning")
    @ApiOperation("保证金预警页面展示")
    public ResultEntity<MarginWarning> getMarginWarning(@ApiParam("姓名") @RequestParam String userName,
                                                        @ApiParam("任务ID") @RequestParam String trainingId,
                                                        @ApiParam("成员ID") @RequestParam String memberId) {

        return new ResultEntity<>(wainingService.getMarginWarning(userName, trainingId, memberId));
    }

    //保证金报警水平保存
    @PostMapping("/saveWarningLevel")
    @ApiOperation("保证金报警水平修改保存")
    public ResultEntity<MarginWarning> saveWarningLevel(@ApiParam("预警水平") @RequestParam Integer warningLevel,
                                                        @ApiParam("成员ID") @RequestParam String memberId,
                                                        @ApiParam("姓名") @RequestParam String userName,
                                                        @ApiParam("任务ID") @RequestParam String trainingId) {
        return new ResultEntity<>(wainingService.saveWarningLevel(warningLevel, memberId, userName, trainingId));
    }

    //查询保证金预警记录
    @GetMapping("/getMarginWarningRecord")
    @ApiOperation("查询保证金预警记录")
    public ResultEntity<PageInfo<MarginWarningRecord>> getMarginWarningRecord(@ApiParam("姓名") @RequestParam String userName,
                                                                              @ApiParam("任务ID") @RequestParam String trainingId,
                                                                              @ApiParam("首页页码") @RequestParam Integer index,
                                                                              @ApiParam("页面条数") @RequestParam Integer size) {
        return new ResultEntity<>(wainingService.getMarginWarningRecord(index, size, userName, trainingId));
    }

    //启用或停止保证金预警状态
    @PutMapping("/setMarginWarningStatus")
    @ApiOperation("启用或停止保证金预警状态")
    public ResultEntity<String> setMarginWarningStatus(@ApiParam("0 停用 1 启用") @RequestParam Integer status,
                                                       @ApiParam("任务ID") @RequestParam String id) {
        wainingService.setMarginWarningStatus(status, id);
        return new ResultEntity<>("修改成功");
    }

    //保证金预警记录表删除
    @DeleteMapping("/deleteMarginWarningRecord")
    @ApiOperation("保证金预警记录表删除")
    public void deleteMarginWarningRecord(@RequestParam String id) {
        marketWarningRecordMapper.deleteByPrimaryKey(id);
    }

    //新增行情预警
    @PostMapping("/addMarketWarning")
    @ApiOperation("新增行情预警")
    public void addMarketWarning(@ApiParam("行情预警对象") @RequestBody MarketWarning marketWarning,
                                 @ApiParam("任务ID") @RequestParam String trainingId,
                                 @ApiParam("姓名") @RequestParam String name) {
        wainingService.addMarketWarning(marketWarning, trainingId, name);
    }

    //查询行情预警
    @GetMapping("/getMarketWarning")
    @ApiOperation("查询行情预警")
    public ResultEntity<PageInfo<MarketWarning>> getMarketWarning(@ApiParam("姓名") @RequestParam String name,
                                                                  @ApiParam("任务ID") @RequestParam String trainingId,
                                                                  @ApiParam("首页页码") @RequestParam Integer index,
                                                                  @ApiParam("页面条数") @RequestParam Integer size) {
        PageHelper.startPage(index, size);
        MarketWarningExample marketWarningExample = new MarketWarningExample();
        marketWarningExample.createCriteria().andNameEqualTo(name).andTariningidEqualTo(trainingId);
        List<MarketWarning> marketWarnings = marketWarningMapper.selectByExample(marketWarningExample);
        PageInfo<MarketWarning> pageInfo = new PageInfo<>(marketWarnings);
        return new ResultEntity<>(pageInfo);
    }

    //查询行情预警记录
    @GetMapping("/getMarketWarningRecord")
    @ApiOperation("查询行情预警记录")
    public ResultEntity<List<MarketWarningRecord>> getMarketWarningRecord(@ApiParam("姓名") @RequestParam String name,
                                                                          @ApiParam("任务ID") @RequestParam String trainingId) {
        MarketWarningRecordExample marketWarningRecordExample = new MarketWarningRecordExample();
        marketWarningRecordExample.createCriteria().andNameEqualTo(name).andTariningidEqualTo(trainingId);
        return new ResultEntity<>(marketWarningRecordMapper.selectByExample(marketWarningRecordExample));
    }

    //删除行情预警
    @DeleteMapping("/deleteMarketWarning")
    @ApiOperation("删除行情预警")
    public void deleteMarketWarningById(@ApiParam("id") @RequestParam String id) {
        marketWarningMapper.deleteByPrimaryKey(id);
    }

    //删除行情预警记录
    @DeleteMapping("/deleteMarketWarningRecord")
    @ApiOperation("删除行情预警记录")
    public void deleteMarketWarningRecordById(@ApiParam("id") @RequestParam String id) {
        marketWarningRecordMapper.deleteByPrimaryKey(id);
    }

    //启用或关闭行情预警
    @PutMapping("/setMarketWarning")
    @ApiOperation("启用或关闭行情预警")
    public String updateStatus(@ApiParam("0 停用 1 启用") @RequestParam Integer status,
                               @ApiParam("id") @RequestParam String id) {
        MarketWarning marketWarning = marketWarningMapper.selectByPrimaryKey(id);
        marketWarning.setStatus(status);
        marketWarningMapper.updateByPrimaryKeySelective(marketWarning);
        return "修改成功";
    }

    //新增持仓时长预警
    @PostMapping("/addAlertHoldDuration")
    @ApiOperation("新增持仓时长预警")
    public void addAlertHoldDuration(@ApiParam("持仓时长预警对象") @RequestBody AlertHoldDuration alertHoldDuration) {
        wainingService.addAlertHoldDuration(alertHoldDuration);
    }

    //删除持仓时长预警
    @DeleteMapping("/deleteAlertHoldDuration")
    @ApiOperation("删除持仓时长预警")
    public void deleteAlertHoldDuration(@ApiParam("id") @RequestParam String id) {
        alertHoldDurationMapper.deleteByPrimaryKey(id);
    }

    //启用或关闭持仓时长预警
    @PutMapping("/setAlertHoldDuration")
    @ApiOperation("启用或关闭持仓时长预警")
    public void setAlertHoldDuration(@ApiParam("0 停用 1 启用") @RequestParam Integer status,
                                     @ApiParam("id") @RequestParam String id) {
        AlertHoldDuration alertHoldDuration = alertHoldDurationMapper.selectByPrimaryKey(id);
        alertHoldDuration.setStatus(status);
        alertHoldDurationMapper.updateByPrimaryKeySelective(alertHoldDuration);
    }

    //删除持仓时长预警记录表
    @DeleteMapping("/deleteAlertHoldDurationRecord")
    @ApiOperation("删除持仓时长预警记录表数据")
    public String deleteAlertHoldDurationRecord(@ApiParam("id") @RequestParam String id) {
        alertHoldDurationRecordMapper.deleteByPrimaryKey(id);
        return "删除成功";
    }

    //查询持仓时长预警记录表
    @GetMapping("/getAlertHoldDurationRecord")
    @ApiOperation("查询持仓时长预警记录表")
    public ResultEntity<List<AlertHoldDurationRecord>> getAlertHoldDurationRecord(@ApiParam("memberId") @RequestParam String memberId,
                                                                                  @ApiParam("memberId") @RequestParam String trainingId) {
        AlertHoldDurationRecordExample alertHoldDurationRecordExample = new AlertHoldDurationRecordExample();
        alertHoldDurationRecordExample.createCriteria().andTariningidEqualTo(trainingId).andMamberidEqualTo(memberId);
        List<AlertHoldDurationRecord> alertHoldDurationRecords = alertHoldDurationRecordMapper.selectByExample(alertHoldDurationRecordExample);
        return new ResultEntity<>(alertHoldDurationRecords);
    }

    //查询持仓时长预警表
    @GetMapping("/getAlertHoldDuration")
    @ApiOperation("查询持仓时长预警表")
    public ResultEntity<List<AlertHoldDuration>> getAlertHoldDuration(@ApiParam("memberId") @RequestParam String memberId,
                                                                      @ApiParam("memberId") @RequestParam String trainingId) {
        AlertHoldDurationExample alertHoldDurationExample = new AlertHoldDurationExample();
        alertHoldDurationExample.createCriteria().andTrainingidEqualTo(trainingId).andMemberidEqualTo(memberId);
        List<AlertHoldDuration> alertHoldDurations = alertHoldDurationMapper.selectByExample(alertHoldDurationExample);
        return new ResultEntity<>(alertHoldDurations);
    }

    //交易次数预警新增
    @PostMapping("/addTransactionNumberWarning")
    @ApiOperation("交易次数预警新增")
    public String addTransactionNumberWarning(@ApiParam("交易次数预警对象") @RequestBody TransactionNumberWarning transactionNumberWarning) {
        return  wainingService.addTransactionNumberWarning(transactionNumberWarning);
    }


    //交易次数预警删除
    @DeleteMapping("/deleteTransactionNumberWarning")
    @ApiOperation("删除持仓时长预警记录表数据")
    public void deleteTransactionNumberWarning(@ApiParam("id") @RequestParam String id) {
        transactionNumberWarningMapper.deleteByPrimaryKey(id);
    }

    //交易次数预警查询
    @GetMapping("/getTransactionNumberWarning")
    @ApiOperation("交易次数预警查询")
    public ResultEntity<List<TransactionNumberWarning>> getTransactionNumberWarning(@ApiParam("memberId") @RequestParam String memberId,
                                                                                    @ApiParam("memberId") @RequestParam String trainingId) {
        TransactionNumberWarningExample tnwr = new TransactionNumberWarningExample();
        tnwr.createCriteria().andTrainingIdEqualTo(trainingId).andMemberIdEqualTo(memberId);
        List<TransactionNumberWarning> tnwrs = transactionNumberWarningMapper.selectByExample(tnwr);
        return new ResultEntity<>(tnwrs);
    }

    //交易次数预警启用或关闭
    @PutMapping("/setTransactionNumberWarning")
    @ApiOperation("启用或关闭持仓时长预警")
    public void setTransactionNumberWarning(@ApiParam("0 停用 1 启用") @RequestParam Integer status,
                                            @ApiParam("id") @RequestParam String id) {
        TransactionNumberWarning alertHoldDuration = transactionNumberWarningMapper.selectByPrimaryKey(id);
        alertHoldDuration.setStatus(status);
        transactionNumberWarningMapper.updateByPrimaryKeySelective(alertHoldDuration);
    }


    //交易次数预警记录查询
    @GetMapping("/getTransactionNumberWarningRecord")
    @ApiOperation("交易次数预警记录查询")
    public ResultEntity<List<TransactionNumberWarningRecord>> getTransactionNumberWarningRecord(@ApiParam("memberId") @RequestParam String memberId,
                                                                                                @ApiParam("memberId") @RequestParam String trainingId) {
        TransactionNumberWarningRecordExample tnwre = new TransactionNumberWarningRecordExample();
        tnwre.createCriteria().andTrainingIdEqualTo(trainingId).andMemberIdEqualTo(memberId);
        List<TransactionNumberWarningRecord> tnwrs = transactionNumberWarningRecordMapper.selectByExample(tnwre);
        return new ResultEntity<>(tnwrs);
    }

    //交易次数预警记录删除
    @DeleteMapping("/deleteTransactionNumberWarningRecord")
    @ApiOperation("删除持仓时长预警记录表数据")
    public void deleteTransactionNumberWarningRecord(@ApiParam("id") @RequestParam String id) {
        transactionNumberWarningRecordMapper.deleteByPrimaryKey(id);
    }
}