package com.sztzjy.forex.trading_trading.service;

import com.sztzjy.forex.trading_trading.config.security.JwtUser;
import com.sztzjy.forex.trading_trading.dto.GradeWeightBO;
import com.sztzjy.forex.trading_trading.dto.GradeWeightDetailBO;
import com.sztzjy.forex.trading_trading.entity.GradeWeight;
import com.sztzjy.forex.trading_trading.entity.GradeWeightExample;
import com.sztzjy.forex.trading_trading.mappers.GradeWeightMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class GradeWeightService {

    @Autowired
    GradeWeightMapper gradeWeightMapper;

    public void insert(GradeWeightBO bo, JwtUser currentUser) {
        GradeWeight.buildGradeWeights(bo, currentUser).forEach(gradeWeightMapper::insertSelective);
    }

    public GradeWeight findBySchoolId(int schoolId) {
        GradeWeightExample example = new GradeWeightExample();
        GradeWeightExample.Criteria criteria = example.createCriteria();
        criteria.andSchoolIdEqualTo(schoolId);
        example.setOrderByClause("create_time desc");

        List<GradeWeight> gradeWeights = gradeWeightMapper.selectByExample(example);
        if (gradeWeights == null || gradeWeights.size() == 0) {
            return null;
        }
        return gradeWeights.get(0);
    }

    public GradeWeight createDefaultWeight(JwtUser user) {
        GradeWeightBO bo = new GradeWeightBO();
        bo.setSimulationTradingProfitScore(50);
        bo.setExperimentalReportScore(50);
        List<GradeWeightDetailBO> details = new ArrayList<>();
        //以原型来看，当收益率30%以上时得分为100或者收益率0%以下得分50，现给个默认值1000或-1000方便数据库区间查询
        details.add(new GradeWeightDetailBO(1000, 30, 100, 1));
        details.add(new GradeWeightDetailBO(30, 10, 80, 2));
        details.add(new GradeWeightDetailBO(10, 0, 60, 3));
        details.add(new GradeWeightDetailBO(0, -1000, 50, 4));
        bo.setDetail(details);
        List<GradeWeight> gradeWeights = GradeWeight.buildGradeWeights(bo, user);
        gradeWeights.forEach(gradeWeightMapper::insertSelective);
        return gradeWeights.get(0);
    }

    public List<GradeWeight> findWeightByWeightId(String weightId) {
        GradeWeightExample example = new GradeWeightExample();
        GradeWeightExample.Criteria criteria = example.createCriteria();
        criteria.andWeightIdEqualTo(weightId);
        example.setOrderByClause("level desc");
        List<GradeWeight> gradeWeights = gradeWeightMapper.selectByExample(example);
        return gradeWeights;
    }

    public GradeWeight findWeightByWeightIdAndYield(String weightId, Double yield) {
        GradeWeightExample example = new GradeWeightExample();
        GradeWeightExample.Criteria criteria = example.createCriteria();
        criteria.andWeightIdEqualTo(weightId);
        criteria.andMaxProfitGreaterThan(yield);
        criteria.andMinProfitLessThanOrEqualTo(yield);
        GradeWeight gradeWeights = gradeWeightMapper.selectByExample(example).get(0);
        return gradeWeights;
    }

    public List<GradeWeight> findWeightBySchoolId(Integer schoolId) {
        GradeWeightExample example = new GradeWeightExample();
        GradeWeightExample.Criteria criteria = example.createCriteria();
        criteria.andSchoolIdEqualTo(schoolId);
        example.setOrderByClause("create_time desc");
        List<GradeWeight> gradeWeightList = gradeWeightMapper.selectByExample(example);
        if (gradeWeightList == null || gradeWeightList.size() == 0) {
            return null;
        }
        GradeWeight gradeWeights = gradeWeightList.get(0);
        return findWeightByWeightId(gradeWeights.getWeightId());
    }

    public Double findReportScore(String id,Double score) {
        GradeWeightExample example = new GradeWeightExample();
        GradeWeightExample.Criteria criteria = example.createCriteria();
        criteria.andWeightIdEqualTo(id);
        List<GradeWeight> gradeWeightList = gradeWeightMapper.selectByExample(example);
        if (gradeWeightList == null||gradeWeightList.size()==0) {
            return null;
        }
        return gradeWeightList.get(0).getReportScoreScale() * score / 100;
    }

}
