package com.sztzjy.forex.trading_trading.service;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.sztzjy.forex.trading_trading.config.security.JwtUser;
import com.sztzjy.forex.trading_trading.dto.MemberVO;
import com.sztzjy.forex.trading_trading.entity.Member;
import com.sztzjy.forex.trading_trading.entity.MemberExample;
import com.sztzjy.forex.trading_trading.mappers.MemberMapper;
import com.sztzjy.forex.trading_trading.util.excel.ExcelData;
import com.sztzjy.forex.trading_trading.util.excel.ExcelProvider;
import com.sztzjy.forex.trading_trading.util.file.IFileUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

@Service
public class MemberService {
    @Autowired
    MemberMapper memberMapper;
    @Autowired
    IFileUtil fileUtil;

    public Member getMemberByMemberIdAndTrainingId(String memberId, String trainingId) {
        MemberExample example = new MemberExample();
        MemberExample.Criteria criteria = example.createCriteria();
        criteria.andMemberIdEqualTo(memberId);
        criteria.andTrainingIdEqualTo(trainingId);
        List<Member> members = memberMapper.selectByExample(example);
        return members.get(0);
    }

    public void updateByPrimaryKey(Member member) {
        memberMapper.updateByPrimaryKey(member);
    }

    public void updateByPrimaryKeySelective(Member member) {
        memberMapper.updateByPrimaryKeySelective(member);
    }

    public Member selectByPrimaryKey(String memberId) {
        Member member = memberMapper.selectByPrimaryKey(memberId);
        return member;
    }

    public void insertAll(List<Member> members) {
        memberMapper.insertAll(members);
    }

    public void insert(Member member) {
        if (checkUser(member.getTrainingId(), member.getStudentNumber())) return;
        memberMapper.insert(member);
    }

    public boolean checkUser(String trainingId, String studentNo) {
        MemberExample example = new MemberExample();
        MemberExample.Criteria criteria = example.createCriteria();
        criteria.andTrainingIdEqualTo(trainingId);
        criteria.andStudentNumberEqualTo(studentNo);
        List<Member> members = memberMapper.selectByExample(example);
        return members.size() > 0;
    }

    public void deleteById(String memberId) {
        memberMapper.deleteByPrimaryKey(memberId);
    }

    public PageInfo<MemberVO> pagedListMembers(String className,
                                               String trainingName,
                                               Integer markStatus,
                                               String keyword,
                                               Integer schoolId,
                                               Integer page,
                                               Integer size) {
        MemberExample example = new MemberExample();
        MemberExample.Criteria criteria = example.createCriteria();
        criteria.andSchoolIdEqualTo(schoolId);
        if (StringUtils.hasText(className)) {
            criteria.andClassGradeEqualTo(className);
        }
        if (StringUtils.hasText(trainingName)) {
            criteria.andTrainingNameEqualTo(trainingName);
        }
        if (markStatus != null) {
            //未评分
            if (markStatus == 0) {
                criteria.andReportScoreIsNull();
            } else {
                //已评分
                criteria.andReportScoreIsNotNull();
            }
        }
        if (StringUtils.hasText(keyword)) {
            criteria.andNameLike("%" + keyword + "%");
            MemberExample.Criteria orCriteria = example.createCriteria();
            orCriteria.andStudentNumberEqualTo("%" + keyword + "%");
            example.or(orCriteria);
        }
        PageHelper.startPage(page, size);
        return new PageInfo<>(memberMapper.pagedListMembers(example));
    }


    public PageInfo<Member> pageListMemberByTrainingId(String trainingId,
                                                       Integer classId,
                                                       String keyword,
                                                       Integer index,
                                                       Integer size) {
        MemberExample example = new MemberExample();
        MemberExample.Criteria criteria = example.createCriteria();
        criteria.andTrainingIdEqualTo(trainingId);
        if (classId!=null&&classId>0) {
           criteria.andClassIdEqualTo(classId);
        }
        if(StringUtils.hasText(keyword)){
            criteria.andNameLike("%"+keyword+"%");
            MemberExample.Criteria orCriteria = example.createCriteria();
            orCriteria.andStudentNumberEqualTo("%"+keyword+"%");
            example.or(orCriteria);
        }
        example.setOrderByClause("stu_rank ASC");
        PageHelper.startPage(index, size);
        return new PageInfo<>(memberMapper.selectByExample(example));
    }


    public void exportMembers(String className,
                              String trainingName,
                              Integer markStatus,
                              String keyword,
                              HttpServletResponse response) throws FileNotFoundException {
        MemberExample example = new MemberExample();
        MemberExample.Criteria criteria = example.createCriteria();
        if (StringUtils.hasText(className)) {
            criteria.andClassGradeEqualTo(className);
        }
        if (StringUtils.hasText(trainingName)) {
            criteria.andTrainingNameEqualTo(trainingName);
        }
        if (markStatus != null) {
            //未评分
            if (markStatus == 0) {
                criteria.andReportScoreIsNull();
            } else {
                //已评分
                criteria.andReportScoreIsNotNull();
            }
        }
        if (StringUtils.hasText(keyword)) {
            criteria.andNameLike("%" + keyword + "%");
            MemberExample.Criteria orCriteria = example.createCriteria();
            orCriteria.andStudentNumberEqualTo("%" + keyword + "%");
            example.or(orCriteria);
        }
        List<MemberVO> members = memberMapper.pagedListMembers(example);
        Assert.notNull(members, "没有可导出数据");
        List<String> titles = Arrays.asList("实训任务名称", "班级", "学号", "姓名", "排名", "累计盈亏", "收益率", "模拟交易实训成绩", "实训报告成绩", "总成绩");
        ExcelData excelData = ExcelData.create(titles);
        for (MemberVO member : members) {
            List<Object> rows = new ArrayList<>();
            rows.add(member.getTrainingName());
            rows.add(member.getClassGrade());
            rows.add(member.getStudentNumber());
            rows.add(member.getName());
            rows.add(member.getStuRank());
            rows.add(member.getCumulativeProfitLoss());
            rows.add(member.getYield());
            rows.add(member.getTradeTrainingScore());
            rows.add(member.getTrainingReportScore());
            rows.add(member.getTotalScore());
            excelData.addRow(rows);
        }
        String fileName = "学生排行榜数据.xlsx";
        String file = fileUtil.getFullPath(fileName);
        FileOutputStream fileOut = new FileOutputStream(file);
        ExcelProvider.SimpleExcelExport(excelData, fileOut);
        fileUtil.download(response, fileName);
        fileUtil.remove(file);
    }

    public List<Map<String, Object>> classScoreTrend(Integer schoolId) {
        return memberMapper.classScoreTrend(schoolId);
    }

    public Member selectByNameAndSchoolIdAndTrainingID(String name, Integer schoolId, String trainingId) {
        MemberExample example = new MemberExample();
        MemberExample.Criteria criteria = example.createCriteria();
        criteria.andSchoolIdEqualTo(schoolId).andNameEqualTo(name).andTrainingIdEqualTo(trainingId);
        List<Member> members = memberMapper.selectByExample(example);
        return members.get(0);
    }

    public List<Map<String, Object>> selectClassListByTrainingId(String trainingId) {
        return memberMapper.selectClassListByTrainingId(trainingId);
    }

    public List<Map<String, Object>> selectTrainingListByClassId(String trainingId) {
        return memberMapper.selectTrainingListByClassId(Integer.valueOf(trainingId));
    }

    public List<Map<String, Object>> selectAllTrainingList(int schoolId) {
        return memberMapper.selectAllTrainingList(schoolId);
    }

    public List<Member> findByTrainingId(String trainingId){
        MemberExample example = new MemberExample();
        example.createCriteria().andTrainingIdEqualTo(trainingId);
        return memberMapper.selectByExample(example);
    }

    public Member findByTrainingIdAndStudentNumber(String trainingId,String studentNumber){
        MemberExample example = new MemberExample();
        example.createCriteria().andTrainingIdEqualTo(trainingId).andStudentNumberEqualTo(studentNumber);
        List<Member> members = memberMapper.selectByExample(example);
        return members.size()>0?members.get(0):null;
    }

    // 分批处理删除业务，防止当studentNumbers过多时生成的SQL语句可能会超过数据库支持的最大长度限制，从而导致SQL执行失败
    public void deleteMembersByTrainingIdNotInStudentInfos(String trainingId, List<String> studentNumbers) {
        int batchSize = 100; // 每个批次的大小
        int totalBatches = (studentNumbers.size() + batchSize - 1) / batchSize;
        for (int batch = 0; batch < totalBatches; batch++) {
            int fromIndex = batch * batchSize;
            int toIndex = Math.min((batch + 1) * batchSize, studentNumbers.size());
            List<String> currentBatch = studentNumbers.subList(fromIndex, toIndex);
            deleteBatchByTrainingIdNotInStudentInfos(trainingId, currentBatch);
        }
    }

    private void deleteBatchByTrainingIdNotInStudentInfos(String trainingId, List<String> studentNumbers) {
        MemberExample example = new MemberExample();
        example.createCriteria().andTrainingIdEqualTo(trainingId).andStudentNumberNotIn(studentNumbers);
        memberMapper.deleteByExample(example);
    }
}
