package com.sztzjy.forex.trading_trading.service;


import cn.hutool.core.util.IdUtil;
import com.alibaba.fastjson.JSONObject;
import com.sztzjy.forex.trading_trading.annotation.AnonymousAccess;
import com.sztzjy.forex.trading_trading.common.mql5API.Mql5API;
import com.sztzjy.forex.trading_trading.config.Constant;
import com.sztzjy.forex.trading_trading.controller.PendingOrderController;
import com.sztzjy.forex.trading_trading.controller.TakeStashController;
import com.sztzjy.forex.trading_trading.controller.TradingController;
import com.sztzjy.forex.trading_trading.entity.ForexMarketData;
import com.sztzjy.forex.trading_trading.entity.mql5Entity.ForexData;
import com.sztzjy.forex.trading_trading.mappers.TrainingMapper;
import com.sztzjy.forex.trading_trading.util.BigDecimalUtils;
import com.sztzjy.forex.trading_trading.util.ForexMarketDateUtil;
import com.sztzjy.forex.trading_trading.util.RedisUtil;
import lombok.extern.slf4j.Slf4j;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.PostMapping;

import javax.annotation.PostConstruct;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.util.*;

/**
 * 定时任务
 */
@Service
@Slf4j
public class ScheduledTask {

    @Autowired
    TrainingMapper trainingMapper;
    @Autowired
    ForexMarketDataService forexMarketDataService;

    @Value("${timer.enable}")
    private boolean timerEnable;

    @Autowired
    RedisUtil redisUtil;

    @Autowired
    TrainingService trainingService;

    @Autowired
    TakeStashController takeStashController;

    @Autowired
    ForexMarketDateUtil forexMarketDateUtil;

    @Autowired
    PendingOrderController pendingOrderController;

    @Autowired
    TradingController tradingController;

    @Autowired
    BigDecimalUtils bigDecimalUtils;

    @Scheduled(cron = "0 * * * * *") // 修改实训状态 每分钟执行一次
    public void updateTrainingStatus() {
        trainingMapper.updateTrainingStatusToInProgress();
        trainingMapper.updateTrainingStatusToEnd();
    }


    //每两分钟执行一次 真实的数据插入
    @PostConstruct
    @Scheduled(cron = "0 */2 * * * ?")
    public void insertForexMarketData() {
//        if(!timerEnable){
//            return;
//        }
        Mql5API api = new Mql5API();
        List<ForexData> forexData = api.getMarketQuotation();
        List<ForexMarketData> forexMarketDataList = ForexMarketData.copyToForexData(forexData);
        forexMarketDataService.insertAll(forexMarketDataList);
        redisUtil.set("ForexDateList", forexMarketDataList);
    }

    //每16秒执行一次 根据真实汇率数据生成模拟汇率数据
    @Scheduled(cron = "0/16 * * * * ?")
    public void insertSimulatedForexMarketData() {
//        if(!timerEnable){
//            return;
//        }

        //获取batchId相同的最后入库的数据
        List<ForexMarketData> forexMarketDataList=redisUtil.get("ForexDateList");
//        forexMarketDataList = forexMarketDataService.selectLastForexMarketData();
        if(forexMarketDataList==null|| forexMarketDataList.size()==0){
            return;
        }
       try {
           Random random = new Random();
           List<ForexMarketData> forexMarketData = new ArrayList<>();
           String batchId = IdUtil.simpleUUID();
           for (ForexMarketData data : forexMarketDataList) {
               Double buyPic = modifyDecimal(data.getBuyPic(), random);
               Double openPriDouble = Double.valueOf(data.getOpenPri());
               String diffAmo = bigDecimalUtils.sub(String.valueOf(buyPic), data.getOpenPri()).toString();
               ForexMarketData resultData = new ForexMarketData(
                       buyPic,
                       data.getClosePri(),
                       data.getCode(),
                       data.getColor(),
                       data.getCurrency(),
                       new Date(),
                       data.getDate(),
                       data.getHighPic(),
                       data.getLowPic(),
                       data.getOpenPri(),
                       data.getRanges(),
                       String.valueOf(modifyDecimal(Double.parseDouble(data.getSellPic()), random)),
                       data.getYesPic(),
                       diffAmo,
                       data.getDiffPer(),
                       batchId);
               forexMarketData.add(resultData);
           }
//           forexMarketDataService.insertAllByMarketData(forexMarketData);
           redisUtil.set("ForexDateList", forexMarketData);
           log.info("--------插入模拟汇率数据成功------");
       }catch (Exception e) {
           log.error("----------插入模拟汇率数据失败：", e.getMessage());
       }
    }


    /**
     * 将小数点后1位之后的数字修改为随机数
     *
     * @param value  原始值
     * @param random 随机数的random 避免多次创建而出现重复值 所以引用实际方法中的random
     * @return 修改后的值
     */
    private static double modifyDecimal(double value, Random random) {
        String stringValue = Double.toString(value);
        int indexOfDecimal = stringValue.indexOf(".");

        if (indexOfDecimal >= 0 && indexOfDecimal + 2 < stringValue.length()) {
            StringBuilder modifiedValue = new StringBuilder(stringValue.substring(0, indexOfDecimal + 2));
            int randomDigit;

            for (int i = indexOfDecimal + 2; i < stringValue.length(); i++) {
                if (i == indexOfDecimal + 2) {
                    modifiedValue.append(stringValue.charAt(i));
                } else {
                    randomDigit = random.nextInt(10);
                    modifiedValue.append(randomDigit);
                }
            }
            value = Double.parseDouble(modifiedValue.toString());
        }
        return value;
    }


    //error
    //监听止损止盈 根据code获取当前买卖价格 如果价格高于/低于止损止盈 则按照止损止盈的值进行平仓
    //查询实训表中状态为1的所有实训并获取ID  获取持仓表中所有状态为0 且止损和止盈不为空的所有数据 获取当前价位如果
    @Scheduled(cron = "0 */2 * * * ?")
    public void monitorStopLossProfit() {
        List<ForexMarketData> forexDateList = redisUtil.get("ForexDateList");
        Map<String,ForexMarketData> forexDateMap = new HashMap();
        for (int i = 0; i < forexDateList.size(); i++) {
            String code = forexDateList.get(i).getCode();
            forexDateMap.put(code,forexDateList.get(i));
        }

        List<String> trainingIdList = trainingService.selectProceedTraining();
        for (int i = 0; i < trainingIdList.size(); i++) {
            Set<String> stashSet = redisUtil.keys("*stashId*");
            Iterator<String> iterator = stashSet.iterator();
            while (iterator.hasNext()) {
                String key = iterator.next();
                Map<Object, Object> map = redisUtil.hmget(key);
                String tradingCode = String.valueOf(map.get("tradingCode"));
                Double stopLoss = (Double) map.get("stopLoss");
                Double stopWin = (Double) map.get("stopWin");
                String stashId = (String) map.get("stashId");
                String memberId = (String) map.get("memberId");
                String trainingId = (String) map.get("trainingId");
                Double transactionVolume = (Double) map.get("transactionVolume");
                Double priceTransaction = (Double) map.get("priceTransaction");
                String buySellType = String.valueOf(map.get("buySellType"));
                ForexMarketData forexMarketData = forexDateMap.get(tradingCode);
                if (tradingCode.startsWith("USD")){ //美元在前
                    Double margin = takeStashController.startUSDMarginNeed(trainingId, memberId, 0, tradingCode, buySellType, transactionVolume);
                    if ("buy".equals(buySellType)) {  //开仓方式为买 则平仓方式为卖 先获取卖价
                        Double sellPic = Double.valueOf(forexMarketData.getSellPic());
                        if(stopLoss!=-1 && sellPic<=stopLoss){
                            Double profitLoss=(stopLoss-priceTransaction)*transactionVolume*Constant.LEVERQUANTITY/stopLoss;  //止损盈利
                            takeStashController.updateMemberAndTakeStash(memberId,stashId,profitLoss,margin);
                        }
                        if(stopWin!=-1 && sellPic>=stopWin){
                            Double profitLoss=(stopWin-priceTransaction)*transactionVolume*Constant.LEVERQUANTITY/stopWin; //止盈盈利
                            takeStashController.updateMemberAndTakeStash(memberId,stashId,profitLoss,margin);
                        }
                    }else { //开仓方式为卖  则平仓方式为买 先获取买价
                        Double buyPic = forexMarketData.getBuyPic();
                        if(stopLoss!=-1 && buyPic<=stopLoss){
                            Double profitLoss=(priceTransaction-stopLoss)*transactionVolume*Constant.LEVERQUANTITY/stopLoss;
                            takeStashController.updateMemberAndTakeStash(memberId,stashId,profitLoss,margin);
                        }
                        if(stopWin!=-1 && buyPic>=stopWin){
                            Double profitLoss=(priceTransaction-stopWin)*transactionVolume*Constant.LEVERQUANTITY/stopWin;
                            takeStashController.updateMemberAndTakeStash(memberId,stashId,profitLoss,margin);
                        }
                    }
                }else { //美元在后
                    Double margin = takeStashController.endUSDMarginNeed( trainingId, memberId,0 , tradingCode, buySellType, transactionVolume, priceTransaction);
                    if ("buy".equals(buySellType)) {  //开仓方式为买 则平仓方式为卖 先获取卖价
                        Double sellPic = Double.valueOf(forexMarketData.getSellPic());
                        if(stopLoss!=-1 && sellPic<=stopLoss){
                            Double profitLoss=(stopLoss-priceTransaction)*transactionVolume*Constant.LEVERQUANTITY;
                            takeStashController.updateMemberAndTakeStash(memberId,stashId,profitLoss,margin);
                        }
                        if(stopWin!=-1 && sellPic>=stopWin){
                            Double profitLoss=(stopWin-priceTransaction)*transactionVolume*Constant.LEVERQUANTITY;
                            takeStashController.updateMemberAndTakeStash(memberId,stashId,profitLoss,margin);
                        }
                    }else { //开仓方式为卖  则平仓方式为买 先获取买价
                        Double buyPic = forexMarketData.getBuyPic();
                        if(stopLoss!=-1 && buyPic<=stopLoss){
                            Double profitLoss=(stopLoss-priceTransaction)*transactionVolume*Constant.LEVERQUANTITY;
                            takeStashController.updateMemberAndTakeStash(memberId,stashId,profitLoss,margin);
                        }
                        if(stopWin!=-1 && buyPic>=stopWin){
                            Double profitLoss=(stopWin-priceTransaction)*transactionVolume*Constant.LEVERQUANTITY;
                            takeStashController.updateMemberAndTakeStash(memberId,stashId,profitLoss,margin);
                        }
                    }
                }
            }
        }

    }


    //挂单自动开仓(到期自动撤单) 定时方法
    @Scheduled(cron = "0 */2 * * * ?")
    public void OpenPendingOrder() {
        Set<String> pendingOrderSet = redisUtil.keys("pengingOrder*");
        List<ForexMarketData> forexDateList = redisUtil.get("ForexDateList");
        Map<String, ForexMarketData> forexMarketDateMap = forexMarketDateUtil.getForexMarketDateMap(forexDateList);

        for (String key : pendingOrderSet) {
            Map<Object, Object> pengingOrderMap = redisUtil.hmget(key);
            Boolean expire = pendingOrderController.isExpire(pengingOrderMap, key); //判断是否到期
            if (expire) {
                continue;
            }
            //判断是否开仓
            Boolean isPpen = pendingOrderController.isOpenTakeStash(pengingOrderMap,forexMarketDateMap);
            if(isPpen){
                continue;
            }
            JSONObject jsonObject = pendingOrderController.pengingKeyReturnJSONObject(pengingOrderMap);
            tradingController.getMarketQuotation(jsonObject);
        }
    }

    //自动清空实例到期的所有redisKey
    @Scheduled(cron = "0 */2 * * * ?")
    public void clearRedisKeyByTrainingId(){
        List<String> trainingIds=trainingMapper.selectTrainingIdByStatus();
        for (int i = 0; i < trainingIds.size(); i++) {
            redisUtil.del("trainingId_"+trainingIds+"*");
        }
    }

}
