package com.sztzjy.forex.trading_trading.service;

import cn.hutool.core.util.IdUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.sztzjy.forex.trading_trading.config.security.JwtUser;
import com.sztzjy.forex.trading_trading.dto.TrainingBO;
import com.sztzjy.forex.trading_trading.entity.Member;
import com.sztzjy.forex.trading_trading.entity.MemberExample;
import com.sztzjy.forex.trading_trading.entity.Training;
import com.sztzjy.forex.trading_trading.entity.TrainingExample;
import com.sztzjy.forex.trading_trading.mappers.MemberMapper;
import com.sztzjy.forex.trading_trading.mappers.TrainingMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;

import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

@Service
public class TrainingService {

    @Autowired
    private TrainingMapper trainingMapper;

    @Autowired
    private MemberMapper memberMapper;


    public Training create(TrainingBO bo, JwtUser currentUser) {
        if (checkTrainingName(bo.getTrainingName(), null)) {
            throw new RuntimeException("实训名称已存在");
        }
        Training training = Training.buildTraining(bo, currentUser);

        trainingMapper.insertSelective(training);
        return training;
    }

    private List<Member> buildMembers(List<Map<String, Object>> list, Training training) {
        List<Member> members = new ArrayList<>();
        if (list != null && list.size() > 0) {
            Member member = new Member();
            member.setMemberId(IdUtil.simpleUUID());
            member.setName(String.valueOf(list.get(0).get("name")));
            member.setTrainingId(training.getTrainingId());
            member.setSchoolId(training.getSchoolId());
            member.setClassGrade(String.valueOf(list.get(0).get("className")));
            member.setMajor(String.valueOf(list.get(0).get("majorName")));
            member.setStudentNumber(String.valueOf(list.get(0).get("studentNo")));
            members.add(member);
        }
        return members;
    }

    public Training findById(String id) {
        return trainingMapper.selectByPrimaryKey(id);
    }

    public void delete(String id) {
        //TODO 删除实训还要删除member表中的trainingId相关数据
        trainingMapper.deleteByPrimaryKey(id);
    }

    public void update(Training training) {
        trainingMapper.updateByPrimaryKey(training);
    }

    public Training update(String id, String name) {
        Training training = findById(id);
        Assert.isTrue(StringUtils.hasText(name), "实训名称不能为空");
        Assert.notNull(training, "该实训不存在");
        Assert.isTrue(!training.getTrainingName().equals(name), "实训名称未发生变化");
        if (checkTrainingName(name, id)) {
            throw new RuntimeException("实训名称已存在");
        }
        training.setTrainingName(name);
        trainingMapper.updateByPrimaryKeySelective(training);
        return training;
    }

    private Boolean checkTrainingName(String name, String id) {
        TrainingExample example = new TrainingExample();
        example.createCriteria().andTrainingNameEqualTo(name);
        if (id != null) {
            example.createCriteria().andTrainingIdNotEqualTo(id);
        }
        List<Training> training = trainingMapper.selectByExample(example);
        return training.size() > 0;
    }

    public PageInfo<Training> pagedListTraining(Integer schoolId, int pageNo, int pageSize) {
        //TODO 待确定过滤参数
        TrainingExample example = new TrainingExample();
        TrainingExample.Criteria criteria = example.createCriteria();
        criteria.andSchoolIdEqualTo(schoolId);
        PageHelper.startPage(pageNo, pageSize);
        return new PageInfo<>(trainingMapper.selectByExample(example));
    }

    public List<Map<String, Object>> findTrainingNameList(JwtUser user) {
        return trainingMapper.selectTrainingNameList(user.getSchoolId());
    }

    // 我的任务列表
    public PageInfo<Training> findTrainingList(Integer index, Integer size, Integer schoolId, String username) {
        PageHelper.startPage(index, size);
        MemberExample memberExample = new MemberExample();
        MemberExample.Criteria criteria = memberExample.createCriteria();
        if (schoolId != null) {
            criteria.andSchoolIdEqualTo(schoolId);
        }
        if (StringUtils.hasText(username)) {
            criteria.andStudentNumberEqualTo(username);
        }
        List<Member> members = memberMapper.selectByExample(memberExample);
        List<Training> trainingList = new ArrayList<>();
        if (!members.isEmpty()) {
            for (Member member : members) {
                String trainingId = member.getTrainingId();
                Training training = trainingMapper.selectByPrimaryKey(trainingId);
                trainingList.add(training);
            }
            return new PageInfo<>(trainingList);
        }
        return new PageInfo<>();
    }


    //查询正在进行中的实训Id
    public List<String> selectProceedTraining() {
        List<String> list = trainingMapper.selectProceedTraining();
        return list;
    }

    public void updatePeopleNum(Boolean mark, String trainingId) {
        if (!mark) {
            trainingMapper.deletePeopleNum(trainingId);
        } else {
            trainingMapper.addPeopleNum(trainingId);
        }
    }

}
