package com.sztzjy.forex.trading_trading.service;

import cn.hutool.core.util.IdUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.sztzjy.forex.trading_trading.config.websocket.WebSocket;
import com.sztzjy.forex.trading_trading.entity.*;
import com.sztzjy.forex.trading_trading.mappers.*;
import com.sztzjy.forex.trading_trading.util.ForexMarketDateUtil;
import com.sztzjy.forex.trading_trading.util.RedisUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import java.sql.Timestamp;
import java.util.*;

@Service
public class WainingService {
    @Autowired
    private MemberMapper memberMapper;
    @Autowired
    private MarginWarningMapper marginWarningMapper;
    @Autowired
    private MarginWarningRecordMapper marginWarningRecordMapper;
    @Autowired
    private MarketWarningMapper marketWarningMapper;
    @Autowired
    private MarketWarningRecordMapper marketWarningRecordMapper;
    @Autowired
    private ForexMarketDateUtil forexMarketDateUtil;
    @Autowired
    private RedisUtil redisUtil;
    @Autowired
    private AlertHoldDurationMapper alertHoldDurationMapper;
    @Autowired
    private AlertHoldDurationRecordMapper alertHoldDurationRecordMapper;
    @Autowired
    private TakeStashMapper takeStashMapper;
    @Autowired
    private TransactionNumberWarningMapper transactionNumberWarningMapper;
    @Autowired
    private TransactionNumberWarningRecordMapper transactionNumberWarningRecordMapper;
    @Autowired
    private WebSocket webSocket;

    //保证金比较接口
    public String compareMarginLevels(String memberId, String trainingId) {
        String message = "保证金已到达预警水平！";
        MemberExample memberExample = new MemberExample();
        memberExample.createCriteria().andMemberIdEqualTo(memberId).andTrainingIdEqualTo(trainingId);
        List<Member> members = memberMapper.selectByExample(memberExample);
        MarginWarningExample marginWarningExample = new MarginWarningExample();
        marginWarningExample.createCriteria().andMemberIdEqualTo(memberId);
        MarginWarningRecordExample mwre = new MarginWarningRecordExample();
        List<MarginWarning> marginWarningList = marginWarningMapper.selectByExample(marginWarningExample);
        for (MarginWarning marginWarning : marginWarningList) {
            for (Member member : members) {
                mwre.createCriteria().andMarginWarningIdEqualTo(marginWarning.getId());
                List<MarginWarningRecord> marginWarningRecords = marginWarningRecordMapper.selectByExample(mwre);
                for (MarginWarningRecord marginWarningRecord : marginWarningRecords) {
                    Double memberLevel = member.getMarginLevel();
                    Integer warningLevel = marginWarning.getWarningLevel();
                    if (memberLevel <= warningLevel) {
                        Date date = new Date();
                        Timestamp current = new Timestamp(date.getTime());
                        marginWarningRecord.setWarningTime(current);
                        marginWarningRecord.setEarlyWarningLevel(memberLevel);
                        marginWarningRecordMapper.updateByPrimaryKeySelective(marginWarningRecord);
                        WebSocket.sendMessage(message);
                    }
                }
            }
        }
        return message;
    }

    //保证金预警查询
    public MarginWarning getMarginWarning(String userName, String trainingId, String memberId) {
        MarginWarning marginWarning = new MarginWarning();
        MarginWarningExample marginWarningExample = new MarginWarningExample();
        marginWarningExample.createCriteria().andNameEqualTo(userName).andMemberIdEqualTo(memberId);
        List<MarginWarning> marginWarningList = marginWarningMapper.selectByExample(marginWarningExample);
        MarginWarningRecordExample marginWarningRecordExample = new MarginWarningRecordExample();
        marginWarningRecordExample.createCriteria().andNameEqualTo(userName).andTrainingidEqualTo(trainingId);
//        如果表里有数据直接返回
        if (marginWarningList != null && !marginWarningList.isEmpty()) {
            for (MarginWarning warning : marginWarningList) {
                return warning;
            }
        }
//        表里没有就从member表里取好封装返回
        MemberExample memberExample = new MemberExample();
        memberExample.createCriteria().andTrainingIdEqualTo(trainingId).andNameEqualTo(userName);
        List<Member> members = memberMapper.selectByExample(memberExample);
        for (Member member : members) {
            marginWarning.setAccount(userName);
            BeanUtils.copyProperties(member, marginWarning);
            marginWarning.setStatus(0);
            marginWarning.setId(IdUtil.simpleUUID());
            marginWarning.setMarginLevel(50.12);
            marginWarning.setNetValue(50.2131);
            marginWarningMapper.insert(marginWarning);
        }
        return marginWarning;
    }

    //保证金预警水平修改保存
    public MarginWarning saveWarningLevel(Integer warningLevel, String memberId, String userName, String trainingId) {
        MarginWarningExample marginWarningExample = new MarginWarningExample();
        marginWarningExample.createCriteria().andNameEqualTo(userName).andMemberIdEqualTo(memberId);
        List<MarginWarning> marginWarningList = marginWarningMapper.selectByExample(marginWarningExample);
        MarginWarningRecord marginWarningRecord = new MarginWarningRecord();

        for (MarginWarning warning : marginWarningList) {
            Date currentDate = new Date();
            Timestamp currentTimestamp = new Timestamp(currentDate.getTime());
            warning.setSetTime(currentTimestamp);
            warning.setStatus(1);
            warning.setAccount(userName);
            warning.setWarningLevel(warningLevel);
            marginWarningMapper.updateByPrimaryKeySelective(warning);
            BeanUtils.copyProperties(warning, marginWarningRecord);
            marginWarningRecord.setId(IdUtil.simpleUUID());
            marginWarningRecord.setMarginWarningId(warning.getId());
            marginWarningRecord.setTrainingid(trainingId);
            marginWarningRecordMapper.insert(marginWarningRecord);
            return warning;
        }
        return null;
    }


    //查询保证金预警记录表
    public PageInfo<MarginWarningRecord> getMarginWarningRecord(Integer index, Integer size, String userName, String trainingId) {
        PageHelper.startPage(index, size);
        MarginWarningRecordExample marginWarningRecordExample = new MarginWarningRecordExample();
        marginWarningRecordExample.createCriteria().andTrainingidEqualTo(trainingId).andNameEqualTo(userName);
        List<MarginWarningRecord> marginWarningRecords = marginWarningRecordMapper.selectByExample(marginWarningRecordExample);
        return new PageInfo<>(marginWarningRecords);
    }

    //启用或者关闭保证金预警
    public void setMarginWarningStatus(Integer status, String id) {
        MarginWarning marginWarning = marginWarningMapper.selectByPrimaryKey(id);
        marginWarning.setStatus((status == 1) ? 1 : 0);
        marginWarningMapper.updateByPrimaryKeySelective(marginWarning);
    }

    //  新增行情预警，并在达到预警值时将数据存入记录表
    public void addMarketWarning(MarketWarning marketWarning, String trainingId, String name) {
        //   记录表需要设置预警水平和预警时间 为报警时候的值
        String marketWarningId = marketWarning.getId();
        MarketWarningRecord marketWarningRecord = new MarketWarningRecord();
        marketWarning.setTime(new Timestamp(new Date().getTime()));
        marketWarning.setTariningid(trainingId);
        marketWarning.setName(name);
        marketWarningMapper.insert(marketWarning);
        BeanUtils.copyProperties(marketWarning, marketWarningRecord);
        marketWarningRecord.setMarketWarningId(marketWarningId);
        marketWarningRecord.setId(IdUtil.simpleUUID());
        marketWarningRecordMapper.insert(marketWarningRecord);
        HashMap<String, Object> objectObjectHashMap = new HashMap<>();
        objectObjectHashMap.put("marketWarningId", marketWarningId);
        redisUtil.hmset("marketWarningRecord_" + marketWarningId + "_", objectObjectHashMap);
        ComparativeTransactionPrice();
    }

    //行情预警，监控用户设置的交易品种当前的交易价格，10S一次
    @Scheduled(fixedRate = 10000)
    public void ComparativeTransactionPrice() {
//       从redis取出所有的marketWarning
        Set<String> marketWarningSet = redisUtil.keys("marketWarningRecord*");
//       从redis取出所有的交易品种的参数
        List<ForexMarketData> forexDateList = redisUtil.get("ForexDateList");
        Map<String, ForexMarketData> forexMarketDateMap = forexMarketDateUtil.getForexMarketDateMap(forexDateList);
//       遍历拿到value 为存入的marketWarningID
        for (String key : marketWarningSet) {
            Map<Object, Object> marketWarningMap = redisUtil.hmget(key);
            String marketWarningId = (String) marketWarningMap.get("marketWarningId");
            MarketWarningRecordExample marketWarningRecordExample = new MarketWarningRecordExample();
            marketWarningRecordExample.createCriteria().andMarketWarningIdEqualTo(marketWarningId);
            List<MarketWarningRecord> marketWarningRecords = marketWarningRecordMapper.selectByExample(marketWarningRecordExample);
            for (MarketWarningRecord marketWarningRecord : marketWarningRecords) {
                ForexMarketData forexMarketData = forexMarketDateMap.get(marketWarningRecord.getSymbol());
                String sellPic = forexMarketData.getSellPic();
                Double buyPic = forexMarketData.getBuyPic();
                Double sellPicValue = Double.parseDouble(sellPic);
//        2.判断新增的预警是买还是卖
                Double upperBreakPrice = marketWarningRecord.getUpperBreakPrice();
                Double lowerBreakPrice = marketWarningRecord.getLowerBreakPrice();
                String direction = marketWarningRecord.getDirection();
                String message = "预警通知：价格已达到预警值";
                if (direction.equals("买")) {
//        3.拿redis的价位和新增的高价低价做比较，==的时候就弹出提示，并且存入行情预警记录表
                    if (buyPic.equals(upperBreakPrice) || buyPic.equals(lowerBreakPrice)) {
                        WebSocket.sendMessage(message);
                        marketWarningRecord.setWarningPrice(buyPic);
                        marketWarningRecord.setWarningTime((new Timestamp(new Date().getTime())));
                        marketWarningRecordMapper.updateByPrimaryKeySelective(marketWarningRecord);
                        redisUtil.del(key);
                    }
                } else {
                    if (sellPicValue.equals(upperBreakPrice) || sellPicValue.equals(lowerBreakPrice)) {
                        WebSocket.sendMessage(message);
                        marketWarningRecord.setWarningPrice(sellPicValue);
                        marketWarningRecord.setWarningTime((new Timestamp(new Date().getTime())));
                        marketWarningRecordMapper.updateByPrimaryKeySelective(marketWarningRecord);
                        redisUtil.del(key);
                    }
                }
            }
        }
    }

    //新增持仓时长预警
    public void addAlertHoldDuration(AlertHoldDuration alertHoldDuration) {
        String symbol = alertHoldDuration.getSymbol();
        TakeStashExample takeStashExample = new TakeStashExample();
        takeStashExample.createCriteria().andTradingCodeEqualTo(symbol);
        List<TakeStash> takeStashes = takeStashMapper.selectByExample(takeStashExample);
        for (TakeStash takeStash : takeStashes) {
            String stashId = takeStash.getStashId();
            alertHoldDuration.setSetTime(new Timestamp(new Date().getTime()));
            alertHoldDuration.setStashid(stashId);
            alertHoldDurationMapper.insert(alertHoldDuration);

            String durationId = alertHoldDuration.getId();
            AlertHoldDurationRecord ahdr = new AlertHoldDurationRecord();
            BeanUtils.copyProperties(alertHoldDuration, ahdr);
            ahdr.setId(IdUtil.simpleUUID());
            ahdr.setDurationid(durationId);
            ahdr.setDirection(takeStash.getBuySellType());
            ahdr.setTradingHour(takeStash.getTimeTransaction());
            HashMap<String, Object> objectObjectHashMap = new HashMap<>();
            objectObjectHashMap.put("takeStashId", stashId);
            objectObjectHashMap.put("durationId", durationId);
            alertHoldDurationRecordMapper.insert(ahdr);

            redisUtil.hmset("AlertHoldDurationTakeStashId" + stashId + "_", objectObjectHashMap);
            b();
        }
    }

    //    持仓时长预警对比定时任务
    @Scheduled(fixedRate = 10000)
    public void b() {
        Set<String> alertHoldDurationTakeStashId = redisUtil.keys("AlertHoldDurationTakeStashId");
        for (String takeStashId : alertHoldDurationTakeStashId) {
            Map<Object, Object> hmget = redisUtil.hmget(takeStashId);
            String resultTakeStashId = (String) hmget.get("takeStashId");
            String durationId = (String) hmget.get("durationId");
            //根据TakeStashId查出对应的持仓记录对象
            AlertHoldDurationRecordExample alertHoldDurationRecordExample = new AlertHoldDurationRecordExample();
            alertHoldDurationRecordExample.createCriteria().andStashidEqualTo(resultTakeStashId);
            TakeStash takeStash = takeStashMapper.selectByPrimaryKey(resultTakeStashId);
            Date timeTransaction = takeStash.getTimeTransaction();  //交易时间
            String buySellType = takeStash.getBuySellType();        //买卖类型
            List<AlertHoldDurationRecord> alertHoldDurationRecords = alertHoldDurationRecordMapper.selectByExample(alertHoldDurationRecordExample);
            AlertHoldDurationRecord ahdr = alertHoldDurationRecords.get(0);

            AlertHoldDuration alertHoldDurations = alertHoldDurationMapper.selectByPrimaryKey(durationId);
            String wainingHoldDuration = alertHoldDurations.getWariningHoldDuration();
            Date setTime = alertHoldDurations.getSetTime();
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(setTime);
            calendar.add(Calendar.DAY_OF_YEAR, Integer.parseInt(wainingHoldDuration));
            //相加后的时间
            Timestamp resultTime = new Timestamp(calendar.getTime().getTime());

            Timestamp nowTime = new Timestamp(new Date().getTime());
            long timeDiff = nowTime.getTime() - timeTransaction.getTime();
            long totalHours = timeDiff / (1000 * 60 * 60);
            long days = totalHours / 24;
            long hours = totalHours % 24;

            String timeString = days + "天 " + hours + "小时";
            //"当前时间等于或大于预警值就将时间存库然后发送消息给前端"
            if (nowTime.compareTo(resultTime) >= 0) {
                ahdr.setWariningTime(nowTime);              //设置预警时间
                ahdr.setTradingHour(timeTransaction);   //设置交易时间
            }
            if (buySellType != null) {
                ahdr.setDirection(buySellType);         //设置买卖类型
            }
            ahdr.setHoldDuration(timeString);             //设置持仓时长
            alertHoldDurationRecordMapper.updateByPrimaryKeySelective(ahdr);
        }
    }

    //新增交易时长预警
    public String addTransactionNumberWarning(TransactionNumberWarning transactionNumberWarning) {
        TransactionNumberWarningRecord tnwr = new TransactionNumberWarningRecord();
        transactionNumberWarning.setSetTime(new Timestamp(new Date().getTime()));

        BeanUtils.copyProperties(transactionNumberWarning, tnwr);
        transactionNumberWarningMapper.insert(transactionNumberWarning);
        transactionNumberWarningRecordMapper.insert(tnwr);
        return "新增成功";
    }

    //比较交易次数的公共方法
    public String CompareTransactionNumber(String memberId, String trainingId) {
        String message = "交易笔数未达到所设置的预警值，请增加交易次数。";
        MemberExample memberExample = new MemberExample();
        memberExample.createCriteria().andMemberIdEqualTo(memberId).andTrainingIdEqualTo(trainingId);
        List<Member> members = memberMapper.selectByExample(memberExample);
        TransactionNumberWarningRecordExample tnwrExample = new TransactionNumberWarningRecordExample();
        tnwrExample.createCriteria().andMemberIdEqualTo(memberId).andTrainingIdEqualTo(trainingId);
        List<TransactionNumberWarningRecord> transactionNumberWarningRecords = transactionNumberWarningRecordMapper.selectByExample(tnwrExample);
        for (TransactionNumberWarningRecord tnwr : transactionNumberWarningRecords) {
            for (Member member : members) {
                Integer warningCount = tnwr.getWarningCount();
                Integer openingTrades = member.getOpeningTrades(); //开仓次数即当前交易次数
                Date warningTime = tnwr.getWarningTime();
                Date date = new Date();
                if (openingTrades < warningCount && date == warningTime) {
                    // 如果当前交易次数在到达用户设置的预警时间时还是小于，那么就返回消息给前端提示
                    tnwr.setWarningingCount(openingTrades);
                    tnwr.setWarningTime(new Timestamp(new Date().getTime()));
                    WebSocket.sendMessage(message);
                    transactionNumberWarningRecordMapper.updateByPrimaryKeySelective(tnwr);
                }
            }
        }
        return message;
    }


//    //  新增行情预警，并在达到预警值时将数据存入记录表
//    public void addMarketWarning(MarketWarning marketWarning, String trainingId, String name) {
//        MarketWarningRecord marketWarningRecord = new MarketWarningRecord();
//        marketWarning.setTime(new Timestamp(new Date().getTime()));
//        marketWarning.setTariningid(trainingId);
//        marketWarning.setName(name);
//        marketWarningMapper.insert(marketWarning);
//        BeanUtils.copyProperties(marketWarning, marketWarningRecord);
//        marketWarningRecord.setMarketWarningId(marketWarning.getId());
//        marketWarningRecord.setId(IdUtil.simpleUUID());
//
////        1.从redis中取出当前价位
//        ForexMarketData forexMarketDateByCode = forexMarketDateUtil.getForexMarketDateByCode(marketWarning.getSymbol());
//        if (forexMarketDateByCode != null) {
//            Double buyPic = forexMarketDateByCode.getBuyPic();
//            String sellPic = forexMarketDateByCode.getSellPic();
//            Double sellPicValue = Double.parseDouble(sellPic);
////        2.判断新增的预警是买还是卖
//            String direction = marketWarning.getDirection();
//            Double upperBreakPrice = marketWarning.getUpperBreakPrice();
//            Double lowerBreakPrice = marketWarning.getLowerBreakPrice();
//            String message = "预警通知：价格已达到预警值";
//            if (direction.equals("买")) {
////        3.拿redis的价位和新增的高价低价做比较，==的时候就弹出提示，并且存入行情预警记录表
//                if (buyPic.equals(upperBreakPrice) || buyPic.equals(lowerBreakPrice)) {
//                    simpMessagingTemplate.convertAndSend("/topic/alerts", message);
//                    marketWarningRecord.setWarningPrice(buyPic);
//                    marketWarningRecord.setWarningTime((new Timestamp(new Date().getTime())));
//                }
//            } else {
//                if (sellPicValue.equals(upperBreakPrice) || sellPicValue.equals(lowerBreakPrice)) {
//                    simpMessagingTemplate.convertAndSend("/topic/alerts", message);
//                    marketWarningRecord.setWarningPrice(sellPicValue);
//                    marketWarningRecord.setWarningTime((new Timestamp(new Date().getTime())));
//                }
//            }
//        }
//        marketWarningRecordMapper.insert(marketWarningRecord);
//    }

//    定时任务，五秒比较一次保证金预警值是否相等，
//    @Scheduled(fixedRate = 10000)
//    public void compareMarginLevels() {
//        Boolean isStorageNeeded = true;
//        Set<String> keys = redisUtil.keys("*marginWarningId*");
//        for (String key : keys) {
//            Map<Object, Object> hmget = redisUtil.hmget(key);
//            Integer warningLevel = (Integer) hmget.get("warningLevel");
//            MarginWarningRecord marginWarningRecord = (MarginWarningRecord) hmget.get("marginWarningRecord");
//            String memberId = (String) hmget.get("memberId");
//            Member member = memberMapper.selectByPrimaryKey(memberId);
//            String message = "保证金已到达预警水平！";
//            if (warningLevel != null) {
//                Double aDouble = Double.valueOf(warningLevel);
//                Double marginLevel = member.getMarginLevel();
//                if (redisUtil.get("StorageOrNot") != null) {
//                    boolean storageOrNot = redisUtil.get("StorageOrNot");
////      相等就存预警水平和预警时间，并且发送消息给前端提醒
//                    if (aDouble >= marginLevel) {
//                        if (!storageOrNot) {
//                            Date date = new Date();
//                            Timestamp current = new Timestamp(date.getTime());
//                            marginWarningRecord.setWarningTime(current);
//                            marginWarningRecord.setWarningLevel(aDouble.intValue());
//                            marginWarningRecordMapper.insert(marginWarningRecord);
//                            redisUtil.set("StorageOrNot", true);
//                            simpMessagingTemplate.convertAndSend("/topic/margin-change", message);
////                        isStorageNeeded = false; // 存完库后设置为false
//                        }
//                    }
//                }
//            }
//        }
//    }
}
